# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import os
import re

import pytest

from cve_bin_tool.cve_scanner import CVEScanner
from cve_bin_tool.error_handler import ErrorMode
from cve_bin_tool.merge import (
    REQUIRED_INTERMEDIATE_METADATA,
    InvalidJsonError,
    MergeReports,
    MissingFieldsError,
)
from cve_bin_tool.util import ProductInfo, Remarks


class TestMergeReports:

    INTERMEDIATE_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)), "json")
    MERGED_TRIAGE_PATH = os.path.join(
        os.path.abspath(os.path.dirname(__file__)), "json"
    )

    MERGED_TRIAGE_DATA = {
        ProductInfo(vendor="libjpeg-turbo", product="libjpeg-turbo", version="2.0.1"): {
            "CVE-2018-19664": {
                "remarks": Remarks.Confirmed,
                "comments": "High priority need to resolve fast",
                "severity": "CRITICAL",
            },
            "paths": {""},
            "CVE-2018-20330": {
                "remarks": Remarks.Unexplored,
                "comments": "Need to mitigate cves of this product",
                "severity": "HIGH",
            },
            "CVE-2020-17541": {
                "remarks": Remarks.Unexplored,
                "comments": "Need to mitigate cves of this product",
                "severity": "HIGH",
            },
        }
    }

    MISSING_FIELD_REGEX = re.compile(r"({.+}) are required fields")

    @pytest.mark.parametrize(
        "filepaths, exception",
        (([os.path.join(INTERMEDIATE_PATH, "bad.json")], InvalidJsonError),),
    )
    def test_invalid_file(self, filepaths, exception):
        merged_cves = MergeReports(
            merge_files=filepaths, error_mode=ErrorMode.FullTrace
        )
        with pytest.raises(exception):
            path = merged_cves.merge_intermediate()

    @pytest.mark.parametrize(
        "filepaths,missing_fields",
        (
            (
                [os.path.join(INTERMEDIATE_PATH, "bad_intermediate.json")],
                {"metadata", "report"},
            ),
            (
                [os.path.join(INTERMEDIATE_PATH, "bad_metadata.json")],
                REQUIRED_INTERMEDIATE_METADATA,
            ),
        ),
    )
    def test_missing_fields(self, filepaths, missing_fields):
        merged_cves = MergeReports(
            merge_files=filepaths, error_mode=ErrorMode.FullTrace
        )
        with pytest.raises(MissingFieldsError) as exc:
            merged = merged_cves.merge_intermediate()
        match = self.MISSING_FIELD_REGEX.search(exc.value.args[0])
        raised_fields = match.group(1)

        assert missing_fields - eval(raised_fields) == set()

    @pytest.mark.parametrize(
        "filepaths, merged_data",
        (
            (
                [os.path.join(INTERMEDIATE_PATH, "test_intermediate.json")],
                MERGED_TRIAGE_DATA,
            ),
        ),
    )
    def test_valid_merge(self, filepaths, merged_data):

        merged_cves = MergeReports(
            merge_files=filepaths, error_mode=ErrorMode.FullTrace, score=0
        )
        merge_cve_scanner = merged_cves.merge_intermediate()
        with CVEScanner(score=0) as cve_scanner:
            for product_info, triage_data in merged_data.items():
                cve_scanner.get_cves(product_info, triage_data)

            assert merge_cve_scanner.all_cve_data == cve_scanner.all_cve_data

    @pytest.mark.parametrize(
        "filepaths",
        (([os.path.join(INTERMEDIATE_PATH, "test_intermediate.json")]),),
    )
    def test_valid_cve_scanner_instance(self, filepaths):

        merged_cves = MergeReports(
            merge_files=filepaths,
            error_mode=ErrorMode.FullTrace,
        )
        merge_cve_scanner = merged_cves.merge_intermediate()

        assert isinstance(merge_cve_scanner, CVEScanner) == True



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "node.js",
        "version": "9.3.0",
        "version_strings": ["https://nodejs.org/download/release/v", "node v9.3.0 "],
    }
]
package_test_data = [
    {
        "url": "http://mirror.centos.org/centos/7/sclo/x86_64/rh/Packages/r/",
        "package_name": "rh-nodejs14-nodejs-14.16.0-1.el7.x86_64.rpm",
        "product": "node.js",
        "version": "14.16.0",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for varnish
https://www.cvedetails.com/vulnerability-list/vendor_id-12937/product_id-26407/Varnish-cache-Varnish.html
"""
from cve_bin_tool.checkers import Checker


class VarnishChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"varnish"]
    VERSION_PATTERNS = [r"varnish-([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("varnish-cache", "varnish")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import json
import textwrap
from urllib import request

from packaging import version

from cve_bin_tool.log import LOGGER

VERSION: str = "2.2.1"


def check_latest_version():
    """Checks for the latest version available at PyPI."""

    name: str = "cve-bin-tool"
    url: str = f"https://pypi.org/pypi/{name}/json"
    try:
        with request.urlopen(url) as resp:
            package_json = json.load(resp)
            pypi_version = package_json["info"]["version"]
            if pypi_version == VERSION:
                LOGGER.info(
                    textwrap.dedent(
                        """
                                *********************************************************
                                Yay! you are running the latest version.
                                But you can try the latest development version at GitHub.
                                URL: https://github.com/intel/cve-bin-tool
                                *********************************************************
                                """
                    )
                )
            else:
                LOGGER.info(
                    f"[bold red]You are running version {VERSION} of {name} but the latest PyPI Version is {pypi_version}.[/]",
                    extra={"markup": True},
                )
                if version.parse(VERSION) < version.parse(pypi_version):
                    LOGGER.info(
                        "[bold yellow]Alert: We recommend using the latest stable release.[/]",
                        extra={"markup": True},
                    )
    except Exception as error:
        LOGGER.warning(
            textwrap.dedent(
                f"""
        -------------------------- Can't check for the latest version ---------------------------
        warning: unable to access 'https://pypi.org/pypi/{name}'
        Exception details: {error}
        Please make sure you have a working internet connection or try again later. 
        """
            )
        )



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "ftp",
        "version": "0.17",
        "version_strings": ["$NetKit: netkit-ftp-0.17 $"],
    }
]
package_test_data = [
    {
        "url": "http://archive.ubuntu.com/ubuntu/pool/main/n/netkit-ftp/",
        "package_name": "ftp_0.17-34.1.1_amd64.deb",
        "product": "ftp",
        "version": "0.17",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

#!/usr/bin/python
""" Validates the NIST data feed
1. Against their schema.
This uses the schemas mentioned here: https://nvd.nist.gov/vuln/Data-Feeds/JSON-feed-changelog
2. Against the provided metadata, including the sha256sum
"""
import datetime
import gzip
import json
import os
import unittest
from test.utils import LONG_TESTS
from urllib.request import urlopen

import pytest
from jsonschema import validate
from jsonschema.exceptions import ValidationError

from cve_bin_tool.cvedb import DISK_LOCATION_DEFAULT
from cve_bin_tool.log import LOGGER

NVD_SCHEMA = "https://scap.nist.gov/schema/nvd/feed/1.1/nvd_cve_feed_json_1.1.schema"
# NVD feeds from "https://nvd.nist.gov/vuln/data-feeds#JSON_FEED" but stored locally


class TestJSON:
    # Download the schema
    SCHEMA = json.loads(urlopen(NVD_SCHEMA).read().decode("utf-8"))
    LOGGER.info("Schema loaded successfully")

    @unittest.skipUnless(LONG_TESTS() > 0, "Skipping long tests")
    @pytest.mark.parametrize(
        "year", list(range(2002, datetime.datetime.now().year + 1))
    )
    # NVD database started in 2002, so range then to now.
    def test_json_validation(self, year):
        """Validate latest nvd json file against their published schema"""
        # Open the latest nvd file on disk
        with gzip.open(
            os.path.join(DISK_LOCATION_DEFAULT, f"nvdcve-1.1-{year}.json.gz"),
            "rb",
        ) as json_file:
            nvd_json = json.loads(json_file.read())
            LOGGER.info(f"Loaded json for year {year}: nvdcve-1.1-{year}.json.gz")

            # Validate -- will raise a ValidationError if not valid
            try:
                validate(nvd_json, self.SCHEMA)
                LOGGER.info("Validation complete")
            except ValidationError as ve:
                LOGGER.error(ve)
                pytest.fail("Validation error occurred")



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for openswan

https://www.cvedetails.com/product/57217/Xelerance-Openswan.html?vendor_id=20146

"""
from cve_bin_tool.checkers import Checker


class OpenswanChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [
        r"klipsdebug",
        r"showhostkey",
        r"ranbits",
        r"eroute",
        r"showpolicy",
        r"spigrp",
        r"pluto",
        r"ikeping",
        r"rsasigkey",
    ]
    VERSION_PATTERNS = [r"Openswan ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("xelerance", "openswan")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "libcurl",
        "version": "7.65.0",
        "version_strings": [
            "An unknown option was passed in to libcurl",
            "CLIENT libcurl 7.65.0",
        ],
    }
]
package_test_data = [
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "libcurl-7.29.0-59.el7.x86_64.rpm",
        "product": "libcurl",
        "version": "7.29.0",
    },
    {
        "url": "https://archives.fedoraproject.org/pub/archive/fedora/linux/releases/30/Everything/x86_64/os/Packages/l/",
        "package_name": "libcurl-7.64.0-6.fc30.x86_64.rpm",
        "product": "libcurl",
        "version": "7.64.0",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "freeradius",
        "version": "2.1.8",
        "version_strings": ["FreeRADIUS Version 2.1.8"],
    }
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/freeradius/3.0.8/2.fc23/x86_64/",
        "package_name": "freeradius-3.0.8-2.fc23.x86_64.rpm",
        "product": "freeradius",
        "version": "3.0.8",
    },
    {
        "url": "http://rpmfind.net/linux/mageia/distrib/5/x86_64/media/core/updates/",
        "package_name": "freeradius-2.2.10-1.mga5.x86_64.rpm",
        "product": "freeradius",
        "version": "2.2.10",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for ftp

https://www.cvedetails.com/vendor/2/?q=FTP

"""
from cve_bin_tool.checkers import Checker


class FtpChecker(Checker):
    CONTAINS_PATTERNS = [
        r"-p: enable passive mode \(default for ftp and pftp\)",
        r"ftp: no answer from ftp-server \(more than 5 sec\).",
    ]
    FILENAME_PATTERNS = [r"ftp"]
    VERSION_PATTERNS = [r"\$NetKit: netkit-ftp-([0-9]+\.[0-9]+) \$"]
    VENDOR_PRODUCT = [("ftp", "ftp")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for Syslog-ng

References:
https://www.cvedetails.com/vulnerability-list/vendor_id-644/product_id-20465/Balabit-Syslog-ng.html
RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=644&product_id=20465&version_id=0&orderby=3&cvssscoremin=0

"""
from cve_bin_tool.checkers import Checker


class SyslogngChecker(Checker):
    CONTAINS_PATTERNS = [
        r"Set syslog-ng control socket",
        r"Stop syslog-ng process",
    ]
    FILENAME_PATTERNS = [r"syslog-ng"]
    VERSION_PATTERNS = [r"syslog-ng-([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("balabit", "syslog-ng"), ("oneidentity", "syslog-ng")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "hostapd", "version": "2.4", "version_strings": ["hostapd v2.4"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/hostapd/2.3/1.fc20/x86_64/",
        "package_name": "hostapd-2.3-1.fc20.x86_64.rpm",
        "product": "hostapd",
        "version": "2.3",
    },
    {
        "url": "http://security.ubuntu.com/ubuntu/pool/universe/w/wpa/",
        "package_name": "hostapd_2.1-0ubuntu1.7_amd64.deb",
        "product": "hostapd",
        "version": "2.1",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "accountsservice",
        "version": "0.6.45",
        "version_strings": ["accounts-daemon 0.6.45"],
    },
    {
        "product": "accountsservice",
        "version": "0.6.55",
        "version_strings": ["accounts-daemon 0.6.55"],
    },
]
package_test_data = [
    {
        "url": "http://ftp.br.debian.org/debian/pool/main/a/accountsservice/",
        "package_name": "accountsservice_0.6.45-2_amd64.deb",
        "product": "accountsservice",
        "version": "0.6.45",
    },
    {
        "url": "http://mirror.centos.org/centos/8/AppStream/aarch64/os/Packages/",
        "package_name": "accountsservice-0.6.55-1.el8.aarch64.rpm",
        "product": "accountsservice",
        "version": "0.6.55",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for cryptsetup

https://www.cvedetails.com/product/35660/?q=Cryptsetup

"""
from cve_bin_tool.checkers import Checker


class CryptsetupChecker(Checker):
    CONTAIN_PATTERNS = [
        r"Legacy offline reencryption already in-progress. Use cryptsetup-reencrypt utility.",
        r"Only LUKS2 format is currently supported. Please use cryptsetup-reencrypt tool for LUKS1.",
    ]
    FILENAME_PATTERNS = [r"cryptsetup"]
    VERSION_PATTERNS = [r"cryptsetup ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("cryptsetup_project", "cryptsetup")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "wireshark",
        "version": "1.10.12",
        "version_strings": ["Wireshark 1.10.12"],
    }
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/wireshark/1.10.13/1.fc20/x86_64/",
        "package_name": "wireshark-1.10.13-1.fc20.x86_64.rpm",
        "product": "wireshark",
        "version": "1.10.13",
    },
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "wireshark-1.10.14-25.el7.i686.rpm",
        "product": "wireshark",
        "version": "1.10.14",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
CVE-bin-tool async util tests
"""
import dataclasses
import subprocess
import unittest.mock
from typing import Callable, Coroutine, Tuple

import pytest

from cve_bin_tool.async_utils import aio_run_command


@dataclasses.dataclass
class FakeProcess:
    returncode: int

    async def communicate(self) -> Tuple[bytes, bytes]:
        return b"", b""


def mkexec(returncode: int) -> Callable[..., Coroutine[None, None, FakeProcess]]:
    async def return_fake_process(*args, **kwargs) -> FakeProcess:
        return FakeProcess(returncode=returncode)

    return return_fake_process


@pytest.mark.asyncio
async def test_aio_run_command_success():
    with unittest.mock.patch("asyncio.create_subprocess_exec", new=mkexec(0)):
        await aio_run_command(("echo", "hello"))


@pytest.mark.asyncio
async def test_aio_run_command_returncode_non_zero():
    with unittest.mock.patch("asyncio.create_subprocess_exec", new=mkexec(1)):
        with pytest.raises(subprocess.CalledProcessError):
            await aio_run_command(("echo", "hello"), process_can_fail=False)



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for P7zip
--------
References:
https://www.cvedetails.com/vulnerability-list/vendor_id-9220/product_id-30936/version_id-204168/7-zip-P7zip-16.02.html
"""
from cve_bin_tool.checkers import Checker


class P7ZipChecker(Checker):
    CONTAINS_PATTERNS = [
        r"Copyright (c) 1999-2020 Igor Pavlov",
    ]
    FILENAME_PATTERNS = [r"7za", r"7z"]
    VERSION_PATTERNS = [r"7-Zip.*? ([0-9]{2}\.[0-9]{2}) : Copyright"]
    VENDOR_PRODUCT = [("7-zip", "p7zip")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for openssh

References:
https://www.cvedetails.com/product/585/Openbsd-Openssh.html?vendor_id=97
"""
from cve_bin_tool.checkers import Checker


class OpensshChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [
        r"scp",
        r"sftp",
        r"ssh-add",
        r"ssh-agent",
        r"ssh-argv0",
        r"ssh-copy-id",
        r"ssh-keygen",
        r"ssh-keyscan",
        r"ssh",
        r"slogin",
        r"sshd",
    ]
    VERSION_PATTERNS = [r"OpenSSH_([0-9]+\.[0-9]+[0-9a-z\s]*)"]
    VENDOR_PRODUCT = [("openbsd", "openssh")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
Resource definitions and helper utilities for tests.
"""
import asyncio
import os
import shutil
import sys
import tempfile
import unittest
from urllib.request import urlopen

import pytest

from cve_bin_tool.async_utils import get_event_loop

CURL_7_20_0_RPM = "curl-7.20.0-4.fc13.x86_64.rpm"
CURL_7_20_0_URL = (
    "https://archives.fedoraproject.org/pub/archive/fedora/linux/releases/13/Everything/x86_64/os/Packages/"
    + CURL_7_20_0_RPM
)
TMUX_DEB_NAME = "tmux_1.8-5_amd64.deb"
TMUX_DEB = "https://mirrors.cat.pdx.edu/ubuntu/pool/main/t/tmux/" + TMUX_DEB_NAME


class TempDirTest:
    """For tests that need a temp directory"""

    @classmethod
    def setup_class(cls):
        cls.tempdir = tempfile.mkdtemp()

    @classmethod
    def teardown_class(cls):
        shutil.rmtree(cls.tempdir)


def download_file(url, target):
    """helper method to download a file"""
    download = urlopen(url)
    with open(target, "wb") as target_file:
        target_file.write(download.read())
    download.close()


def LONG_TESTS():
    LONG_TESTS = 0
    # override LONG_TESTS with environment variable if available
    if os.getenv("LONG_TESTS"):
        LONG_TESTS = int(os.getenv("LONG_TESTS"))
    return LONG_TESTS


@pytest.fixture
def event_loop():
    yield get_event_loop()



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "icecast",
        "version": "2.3.0",
        "version_strings": ["Icecast 2.3.0"],
    }
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/icecast/2.3.1/4.fc7/x86_64/",
        "package_name": "icecast-2.3.1-4.fc7.x86_64.rpm",
        "product": "icecast",
        "version": "2.3.1",
    },
    {
        "url": "http://www.rpmfind.net/linux/fedora/linux/development/rawhide/Everything/aarch64/os/Packages/i//",
        "package_name": "icecast-2.4.4-6.fc34.aarch64.rpm",
        "product": "icecast",
        "version": "2.4.4",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "gstreamer",
        "version": "1.10.0",
        "version_strings": [
            "http://bugzilla.gnome.org/enter_bug.cgi?product=GStreamer",
            "libgstreamer-1.10.0",
        ],
    }
]
package_test_data = [
    {
        "url": "http://archive.ubuntu.com/ubuntu/pool/universe/g/gstreamermm-1.0/",
        "package_name": "libgstreamermm-1.0-0v5_1.4.3+dfsg-5_amd64.deb",
        "product": "gstreamer",
        "version": "1.0",
    },
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "gstreamer-0.10.36-7.el7.i686.rpm",
        "product": "gstreamer",
        "version": "0.10",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import asyncio
import datetime
import shutil
import sys
import tempfile
from test.utils import event_loop

import aiohttp
import pytest

from cve_bin_tool.cvedb import CVEDB


class TestCVEDB:
    @classmethod
    def setup_class(cls):
        cls.cvedb = CVEDB(cachedir=tempfile.mkdtemp(prefix="cvedb-"))

    @classmethod
    def teardown_class(cls):
        shutil.rmtree(cls.cvedb.cachedir)

    @pytest.mark.asyncio
    async def test_00_getmeta(self):
        connector = aiohttp.TCPConnector(limit_per_host=19)
        async with aiohttp.ClientSession(
            connector=connector, trust_env=True
        ) as session:
            _jsonurl, meta = await self.cvedb.getmeta(
                session,
                "https://nvd.nist.gov/feeds/json/cve/1.1/nvdcve-1.1-modified.meta",
            )
        assert "sha256" in meta

    @pytest.mark.asyncio
    async def test_01_nist_scrape(self):
        async with aiohttp.ClientSession(trust_env=True) as session:
            jsonshas = await self.cvedb.nist_scrape(session)
            assert (
                "https://nvd.nist.gov/feeds/json/cve/1.1/nvdcve-1.1-2015.json.gz"
                in jsonshas
            )

    @pytest.mark.asyncio
    async def test_02_cache_update(self):
        async with aiohttp.ClientSession(trust_env=True) as session:
            jsonurl, meta = await self.cvedb.getmeta(
                session, "https://nvd.nist.gov/feeds/json/cve/1.1/nvdcve-1.1-2015.meta"
            )
            assert "sha256" in meta
            await self.cvedb.cache_update(session, jsonurl, meta["sha256"])

    @pytest.mark.asyncio
    async def test_03_refresh(self):
        await self.cvedb.refresh()
        years = self.cvedb.nvd_years()
        for year in range(2002, datetime.datetime.now().year):
            assert year in years, f"Missing NVD data for {year}"



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for openssl

References:
https://www.openssl.org/news/vulnerabilities.html
http://www.cvedetails.com/vulnerability-list/vendor_id-217/product_id-383/Openssl-Openssl.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=217&product_id=383&version_id=&orderby=3&cvssscoremin=0
"""
from cve_bin_tool.checkers import Checker


class OpensslChecker(Checker):
    CONTAINS_PATTERNS = [r"part of OpenSSL", r"openssl.cnf", r"-DOPENSSL_"]
    FILENAME_PATTERNS = [r"libssl.so.", r"libcrypto.so"]
    VERSION_PATTERNS = [
        r"part of OpenSSL ([01]+\.[0-9]+\.[0-9]+[a-z]*) ",
        r"OpenSSL ([01]+\.[0-9]+\.[0-9]+[a-z]*) ",
    ]
    VENDOR_PRODUCT = [("openssl", "openssl")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "glibc",
        "version": "2.31",
        "version_strings": [
            "GLIBC 2.31",
            "The following command substitution is needed to make ldd work in SELinux",
            "environments where the RTLD might not have permission to write to the",
        ],
    }
]
package_test_data = [
    # works locally, no luck in CI
    # {
    #    "url": "https://kojipkgs.fedoraproject.org/packages/glibc/2.28/39.fc29/i686/",
    #    "package_name": "glibc-2.28-39.fc29.i686.rpm",
    #    "product": "glibc",
    #    "version": "2.28",
    # },
    {
        "url": "https://rpmfind.net/linux/fedora/linux/releases/33/Everything/x86_64/os/Packages/g/",
        "package_name": "glibc-2.32-1.fc33.i686.rpm",
        "product": "glibc",
        "version": "2.32",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "openldap",
        "version": "2.4.37",
        "version_strings": ["ldapsearch 2.4.37"],
    }
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/openldap/2.4.34/1.fc18/x86_64/",
        "package_name": "openldap-clients-2.4.34-1.fc18.x86_64.rpm",
        "product": "openldap",
        "version": "2.4.34",
    },
    {
        "url": "https://mirrors.kernel.org/mageia/distrib/5/x86_64/media/core/release/",
        "package_name": "openldap-clients-2.4.40-3.mga5.x86_64.rpm",
        "product": "openldap",
        "version": "2.4.40",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import csv
import json
import os
import time
from logging import Logger
from typing import IO, Dict, List, Union

from ..cve_scanner import CVEData
from ..cvedb import CVEDB
from ..error_handler import ErrorHandler, ErrorMode
from ..log import LOGGER
from ..util import ProductInfo
from ..version import VERSION
from . import pdfbuilder
from .console import output_console
from .html import output_html
from .util import (
    add_extension_if_not,
    format_output,
    generate_filename,
    intermediate_output,
)


def output_json(all_cve_data: Dict[ProductInfo, CVEData], outfile: IO):
    """Output a JSON of CVEs"""
    formatted_output = format_output(all_cve_data)
    json.dump(formatted_output, outfile, indent="    ")


def save_intermediate(
    all_cve_data: Dict[ProductInfo, CVEData],
    filename: str,
    tag: str,
    scanned_dir: str,
    products_with_cve: int,
    products_without_cve: int,
    total_files: int,
):
    """Save the intermediate report"""

    inter_output = intermediate_output(
        all_cve_data,
        tag,
        scanned_dir,
        products_with_cve,
        products_without_cve,
        total_files,
    )
    with open(filename, "w") as f:
        json.dump(inter_output, f, indent="    ")


def output_csv(all_cve_data: Dict[ProductInfo, CVEData], outfile):
    """Output a CSV of CVEs"""
    formatted_output = format_output(all_cve_data)
    writer = csv.DictWriter(
        outfile,
        fieldnames=[
            "vendor",
            "product",
            "version",
            "cve_number",
            "severity",
            "score",
            "cvss_version",
            "paths",
            "remarks",
            "comments",
        ],
    )
    writer.writeheader()
    writer.writerows(formatted_output)


def output_pdf(
    all_cve_data: Dict[ProductInfo, CVEData], is_report, products_with_cve, outfile
):
    """Output a PDF of CVEs"""
    cvedb_data = CVEDB()
    db_date = time.strftime(
        "%d %B %Y at %H:%M:%S", time.localtime(cvedb_data.get_db_update_date())
    )
    app_version = VERSION
    # Build document
    pdfdoc = pdfbuilder.PDFBuilder()
    cm = pdfdoc.cm
    severity_colour = {
        "UNKNOWN": pdfdoc.grey,
        "LOW": pdfdoc.blue,
        "MEDIUM": pdfdoc.green,
        "HIGH": pdfdoc.orange,
        "CRITICAL": pdfdoc.red,
    }
    pdfdoc.front_page("Vulnerability Report")
    pdfdoc.heading(1, "Introduction")
    pdfdoc.paragraph(
        "The identification of vulnerabilities has been performed using cve-bin-tool version "
        + app_version
    )
    pdfdoc.paragraph(
        "The data used has been obtained from the NVD database which was retrieved on "
        + db_date
        + " and contained "
        + str(cvedb_data.get_cve_count())
        + " entries."
    )

    if is_report:
        pdfdoc.heading(1, "List of All Scanned binaries")
        pdfdoc.createtable(
            "Productlist",
            ["Vendor", "Product", "Version"],
            pdfdoc.tblStyle,
        )
        row = 1
        for product_info, cve_data in all_cve_data.items():
            star_warn = True if "*" in product_info.vendor else False
            for cve in cve_data["cves"]:
                entry = [
                    product_info.vendor,
                    product_info.product,
                    product_info.version,
                ]
                pdfdoc.addrow(
                    "Productlist",
                    entry,
                )
                row += 1
        pdfdoc.showtable("Productlist", widths=[3 * cm, 2 * cm, 2 * cm])
        pdfdoc.paragraph("* vendors guessed by the tool") if star_warn else None
        pdfdoc.paragraph(
            f"There are {products_with_cve} products with vulnerabilities found."
        )
        pdfdoc.pagebreak()

    if products_with_cve != 0:
        pdfdoc.heading(1, "List of Identified Vulnerabilities")
        pdfdoc.paragraph(
            "The following vulnerabilities are reported against the identified versions of the libraries."
        )
        pdfdoc.createtable(
            "Productlist",
            ["Vendor", "Product", "Version", "CVE Number", "Severity"],
            pdfdoc.tblStyle,
            [10, 10, None, None, None],
        )
        row = 1
        star_warn = False
        for product_info, cve_data in all_cve_data.items():
            for cve in cve_data["cves"]:
                if cve.cve_number != "UNKNOWN":
                    if "*" in product_info.vendor:
                        star_warn = True
                    entry = [
                        product_info.vendor,
                        product_info.product,
                        product_info.version,
                        cve.cve_number,
                        cve.severity,
                    ]
                    pdfdoc.addrow(
                        "Productlist",
                        entry,
                        [
                            (
                                "TEXTCOLOR",
                                (3, row),
                                (4, row),
                                severity_colour[cve.severity],
                            ),
                            ("FONT", (3, row), (4, row), "Helvetica-Bold"),
                        ],
                    )
                    row += 1

        pdfdoc.showtable("Productlist", widths=[3 * cm, 3 * cm, 2 * cm, 4 * cm, 3 * cm])
    pdfdoc.paragraph("* vendors guessed by the tool") if star_warn else None

    pdfdoc.pagebreak()
    pdfdoc.paragraph("END OF DOCUMENT.")
    pdfdoc.publish(outfile)


class OutputEngine:
    def __init__(
        self,
        all_cve_data: Dict[ProductInfo, CVEData],
        scanned_dir: str,
        filename: str,
        themes_dir: str,
        time_of_last_update,
        tag: str,
        logger: Logger = None,
        products_with_cve: int = 0,
        products_without_cve: int = 0,
        total_files: int = 0,
        is_report: bool = False,
        append: Union[str, bool] = False,
        merge_report: Union[None, List[str]] = None,
    ):
        self.logger = logger or LOGGER.getChild(self.__class__.__name__)
        self.all_cve_data = all_cve_data
        self.scanned_dir = scanned_dir
        self.filename = os.path.abspath(filename) if filename else ""
        self.products_with_cve = products_with_cve
        self.products_without_cve = products_without_cve
        self.total_files = total_files
        self.themes_dir = themes_dir
        self.is_report = is_report
        self.time_of_last_update = time_of_last_update
        self.append = append
        self.tag = tag
        self.merge_report = merge_report

    def output_cves(self, outfile, output_type="console"):
        """Output a list of CVEs
        format self.checkers[checker_name][version] = dict{id: severity}
        to other formats like CSV or JSON
        """
        if output_type == "json":
            output_json(self.all_cve_data, outfile)
        elif output_type == "csv":
            output_csv(self.all_cve_data, outfile)
        elif output_type == "pdf":
            output_pdf(
                self.all_cve_data, self.is_report, self.products_with_cve, outfile
            )
        elif output_type == "html":
            output_html(
                self.all_cve_data,
                self.scanned_dir,
                self.filename,
                self.themes_dir,
                self.total_files,
                self.products_with_cve,
                self.products_without_cve,
                self.merge_report,
                self.logger,
                outfile,
            )
        else:  # console, or anything else that is unrecognised
            output_console(self.all_cve_data, self.time_of_last_update)

        if isinstance(self.append, str):
            save_intermediate(
                self.all_cve_data,
                self.append,
                self.tag,
                self.scanned_dir,
                self.products_with_cve,
                self.products_without_cve,
                self.total_files,
            )
            self.logger.info(f"Output stored at {self.append}")

    def output_file(self, output_type="console"):

        """Generate a file for list of CVE"""
        if self.append:
            if isinstance(self.append, str):
                self.append = self.check_dir_path(
                    self.append, output_type="json", prefix="intermediate"
                )
                self.append = add_extension_if_not(self.append, "json")
                self.append = self.check_file_path(
                    self.append, output_type="json", prefix="intermediate"
                )
            else:
                # file path for intermediate report not given
                self.append = generate_filename("json", "intermediate")

        if output_type == "console":
            # short circuit file opening logic if we are actually
            # just writing to stdout
            self.output_cves(self.filename, output_type)
            return

        # Check if we need to generate a filename
        if not self.filename:
            self.filename = generate_filename(output_type)
        else:
            # check and add if the filename doesn't contain extension
            self.filename = add_extension_if_not(self.filename, output_type)

            self.filename = self.check_file_path(self.filename, output_type)

            # try opening that file
            with ErrorHandler(mode=ErrorMode.Ignore) as e:
                with open(self.filename, "w") as f:
                    f.write("testing")
                os.remove(self.filename)
            if e.exit_code:
                self.logger.info(
                    f"Exception {e.exc_val} occurred while writing to the file {self.filename} "
                    "Switching Back to Default Naming Convention"
                )
                self.filename = generate_filename(output_type)

        # Log the filename generated
        if output_type == "html" or output_type == "pdf":
            self.logger.info(f"{output_type.upper()} report stored at {self.filename}")
        else:
            self.logger.info(f"Output stored at {self.filename}")

        # call to output_cves
        mode = "w"
        if output_type == "pdf":
            mode = "wb"
        with open(self.filename, mode) as f:
            self.output_cves(f, output_type)

    def check_file_path(self, filepath: str, output_type: str, prefix: str = "output"):
        # check if the file already exists
        if os.path.isfile(filepath):
            self.logger.warning(f"Failed to write at '{filepath}'. File already exists")
            self.logger.info("Generating a new filename with Default Naming Convention")
            filepath = generate_filename(output_type, prefix)

        return filepath

    def check_dir_path(
        self, filepath: str, output_type: str, prefix: str = "intermediate"
    ):

        if os.path.isdir(filepath):
            self.logger.info(
                f"Generating a new filename with Default Naming Convention in directory path {filepath}"
            )
            filename = os.path.basename(generate_filename(output_type, prefix))
            filepath = os.path.join(filepath, filename)

        return filepath



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for kerberos (CLI/library)

References:
https://www.cvedetails.com/vulnerability-list/vendor_id-42/product_id-61/MIT-Kerberos.html
"""
from cve_bin_tool.checkers import Checker


class KerberosChecker(Checker):
    CONTAINS_PATTERNS = [r"KRB5_BRAND: "]
    FILENAME_PATTERNS = [r"kerberos"]
    VERSION_PATTERNS = [
        r"KRB5_BRAND: krb5-(\d+\.\d+\.?\d?)-final",
        r"kerberos 5[_-][apl-]*(1+\.[0-9]+(\.[0-9]+)*)",
    ]
    VENDOR_PRODUCT = [("mit", "kerberos"), ("mit", "kerberos_5")]

    def get_version(self, lines, filename):
        version_info = super().get_version(lines, filename)

        # currently we're only detecting kerberos 5, so return a double-version_info list
        # if we ever detect kerberos that's not 5, this if statement will change
        if "is_or_contains" in version_info:
            version_info5 = [dict(), dict()]
            version_info5[0] = version_info
            version_info5[1] = dict()
            version_info5[1]["is_or_contains"] = version_info["is_or_contains"]
            version_info5[1]["productname"] = "kerberos_5"

            # strip the leading "5-" off the version for 'kerberos_5' if there is one
            # or conversely, add one to the 'kerberos' listing if there isn't
            if version_info["version"][:2] == "5-":
                version_info5[1]["version"] = version_info["version"][2:]
            else:
                version_info5[1]["version"] = version_info["version"]
                version_info5[0]["version"] = "5-{}".format(version_info["version"])
            return version_info5

        return version_info



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "bash", "version": "1.14.0", "version_strings": ["Bash version 1.14.0"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/bash/4.0/1.fc11/x86_64/",
        "package_name": "bash-4.0-1.fc11.x86_64.rpm",
        "product": "bash",
        "version": "4.0.0",
    },
    {
        "url": "http://mirrors.cat.pdx.edu/ubuntu/pool/main/b/bash/",
        "package_name": "bash_4.4.18-2ubuntu1_amd64.deb",
        "product": "bash",
        "version": "4.4.19",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for icu CLI

References:
https://www.cvedetails.com/vulnerability-list/vendor_id-17477/Icu-project.html
"""
from cve_bin_tool.checkers import Checker


class IcuChecker(Checker):
    CONTAINS_PATTERNS = [
        r"-i or --icudatadir       directory for locating any needed intermediate data files,",
        r"-j or --write-java       write a Java ListResourceBundle for ICU4J, followed by optional encoding",
        r"failed to load root collator \(ucadata.icu\) - %s",
        r"is about 300kB larger than the ucadata-implicithan\.icu version\.",
        r"the ucadata-unihan\.icu version of the collation root data",
    ]
    FILENAME_PATTERNS = [r"genrb", r"uconv"]
    VERSION_PATTERNS = [
        r"icu-([0-9]+\.[0-9]+(\.[0-9]+)?)",
        r"ICU ([0-9]+\.[0-9]+(\.[0-9]+)?)",
    ]
    VENDOR_PRODUCT = [("icu-project", "international_components_for_unicode")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "ffmpeg",
        "version": "4.1.3",
        "version_strings": [
            "%s version 4.1.3",
            "Codec '%s' is not recognized by FFmpeg.",
        ],
    }
]
package_test_data = [
    {
        "url": "http://archive.ubuntu.com/ubuntu/pool/universe/f/ffmpeg/",
        "package_name": "ffmpeg_4.3.1-4ubuntu1_amd64.deb",
        "product": "ffmpeg",
        "version": "4.3.1",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for dovecot

https://www.cvedetails.com/product/10948/Dovecot-Dovecot.html?vendor_id=6485

"""
from cve_bin_tool.checkers import Checker


class DovecotChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"dovecot"]
    VERSION_PATTERNS = [r"Dovecot v([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("dovecot", "dovecot")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

#!/usr/bin/python3
import os
import sys

from cve_bin_tool import cli
from cve_bin_tool.error_handler import ErrorHandler, InsufficientArgs, excepthook
from cve_bin_tool.log import LOGGER

sys.excepthook = excepthook


def main(argv=None):
    logger = LOGGER.getChild("CSV2CVE")
    argv = argv or sys.argv
    if len(argv) < 2:
        with ErrorHandler(logger=logger):
            raise InsufficientArgs("csv file required")

    flag = False
    for idx, arg in enumerate(argv):
        if arg.endswith(".csv"):
            argv[idx] = f"-i={arg}"
            flag = True
    if flag:
        return cli.main(argv)
    else:
        with ErrorHandler(logger=logger):
            raise InsufficientArgs("csv file required")


if __name__ == "__main__":
    if os.getenv("NO_EXIT_CVE_NUM"):
        main()
    else:
        sys.exit(main())



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for binutils

References:
https://www.gnu.org/software/binutils/
https://www.cvedetails.com/vulnerability-list/vendor_id-72/product_id-6825/GNU-Binutils.html
"""

from cve_bin_tool.checkers import Checker


class BinutilsChecker(Checker):
    CONTAINS_PATTERNS = [
        # for all programs - special alternate for ubuntu
        r"\(GNU Binutils\) ",
        r"\(GNU Binutils for Ubuntu\) ",
        # bfd
        r"Auxiliary filter for shared object symbol table",
        # ld - the GNU linker.
        r"Do not copy DT_NEEDED links mentioned inside DSOs that follow",
        r"%F%P:%S: error: memory region `%s' for alias `%s' does not exist",
        # as - the GNU assembler.
        r"can't mix positional and keyword arguments",
        r".bundle_lock sequence at %u bytes but .bundle_align_mode limit is %u bytes",
        # addr2line - Converts addresses into filenames and line numbers.
        r"%s: error: too many @-files encountered",
        r"Warning: '%s' has negative size, probably it is too large",
        # ar - A utility for creating, modifying and extracting from archives.
        r"Cannot convert existing library %s to thin format",
        # c++filt - Filter to demangle encoded C++ symbols.
        r"Internal error: no symbol alphabet for current style",
        # dlltool - Creates files for building and using DLLs.
        # gold - A new, faster, ELF only linker, still in beta test.
        r"restart link with --incremental-full",
        # gprof - Displays profiling information.
        r"%s: gmon.out file is missing histogram",
        # nlmconv - Converts object code into an NLM.
        # nm - Lists symbols from object files.
        r"Using the --size-sort and --undefined-only options together",
        # objcopy - Copies and translates object files.
        r"alloc, load, noload, readonly, debug, code, data, rom, share, contents, merge, strings",
        # objdump - Displays information from object files.
        # ranlib - Generates an index to the contents of an archive.
        # readelf - Displays information from any ELF format object file.
        # size - Lists the section sizes of an object or archive file.
        # strings - Lists printable strings from files.
        r"can't set BFD default target to `%s': %s"
        # strip - Discards symbols.
        # windmc - A Windows compatible message compiler.get_ver
        # windres - A compiler for Windows resource files.
    ]
    FILENAME_PATTERNS = [
        # check if it's the library instead of the command line utils
        r"libbfd-((\d+\.)*\d+)[\d\w.-]*so",
        # command line utils
        r"^ld$",  # the GNU linker.
        r"^as$",  # the GNU assembler.
        r"addr2line",  # Converts addresses into filenames and line numbers.
        r"^ar$",  # A utility for creating, modifying and extracting from archives.
        r"c\+{2}filt",  # Filter to demangle encoded C++ symbols.
        r"dlltool",  # Creates files for building and using DLLs.
        r"^gold$",  # A new, faster, ELF only linker, still in beta test.
        r"gprof",  # Displays profiling information.
        r"nlmconv",  # Converts object code into an NLM.
        r"^nm$",  # Lists symbols from object files.
        r"objcopy",  # Copies and translates object files.
        r"objdump",  # Displays information from object files.
        r"ranlib",  # Generates an index to the contents of an archive.
        r"readelf",  # Displays information from any ELF format object file.
        r"^size$",  # Lists the section sizes of an object or archive file.
        r"^strings$",  # Lists printable strings from files.
        r"^strip$",  # Discards symbols.
        r"windmc",  # A Windows compatible message compiler.
        r"windres",  # A compiler for Windows resource files.
        # distro-specific names
        r"ld.bfd",  # as seen on ubuntu
    ]
    VERSION_PATTERNS = [r"libbfd-((\d+\.)*\d+)[\d\w.-]*so"]
    VENDOR_PRODUCT = [("gnu", "binutils")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for mysql CLI
References:
https://www.mysql.com/
https://www.cvedetails.com/vulnerability-list/vendor_id-93/product_id-21801/Oracle-Mysql.html
"""
from cve_bin_tool.checkers import Checker


class MysqlChecker(Checker):

    CONTAINS_PATTERNS = [
        r"To buy MySQL Enterprise support, training, or other products, visit:",
    ]
    FILENAME_PATTERNS = [
        r"mysql",
        r"mysqladmin",
        r"mysql-community",
        r"mysql-client",
        r"mysql-client-core",
        r"mysql-server",
        r"mysql-server-core",
        r"mysql-community-client",
        r"mysql-bench",
        r"dovecot-mysql",
    ]
    VERSION_PATTERNS = [r"mysql\-[0-9]+\.[0-9]+\-([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("oracle", "mysql"), ("mysql", "mysql")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for libcurl

References:
https://curl.haxx.se/docs/security.html
http://www.cvedetails.com/vulnerability-list/vendor_id-12682/Haxx.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=12682&product_id=0&version_id=0&orderby=3&cvssscoremin=0

Note: Some of the "first vulnerable in" data may not be entered correctly.
"""
from cve_bin_tool.checkers import Checker


class LibcurlChecker(Checker):
    CONTAINS_PATTERNS = [
        r"An unknown option was passed in to libcurl",
        r"A requested feature, protocol or option was not found built-in in this libcurl due to a build-time decision.",
        r"CLIENT libcurl 7.",
    ]
    FILENAME_PATTERNS = [r"libcurl.so."]
    VERSION_PATTERNS = [r"CLIENT libcurl ([678]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("haxx", "curl"), ("haxx", "libcurl")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for haproxy

https://www.cvedetails.com/product/22372/Haproxy-Haproxy.html?vendor_id=11969

"""
from cve_bin_tool.checkers import Checker


class HaproxyChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"haproxy"]
    VERSION_PATTERNS = [r"HA-Proxy version ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("haproxy", "haproxy")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
output_engine/util.py - Provides helper functions for OutputEngine.
"""

import os
from collections import defaultdict
from datetime import datetime
from typing import DefaultDict, Dict, List, Union

from ..util import CVE, CVEData, ProductInfo, Remarks


def generate_filename(extension: str, prefix: str = "output") -> str:
    """
    summary: Generate a unique filename with extension provided.
    Function use inbuilt datetime function to generate unique filename.

    Args:
        extension str : Can be any value from list[CSV, JSON, HTML]

    Returns:
        str: unique generated filename
    """
    now = datetime.now().strftime("%Y-%m-%d.%H-%M-%S")

    filename = os.path.abspath(
        os.path.join(os.getcwd(), f"{prefix}.cve-bin-tool.{now}.{extension}")
    )

    return filename


def format_output(all_cve_data: Dict[ProductInfo, CVEData]) -> List[Dict[str, str]]:
    """
    summary: format output in the list of dictionary format.

    Returns:
        formatted_output: List[Dict[str, str]]
        - example:  [
                        {
                            "vendor": "haxx"
                            "product": "curl",
                            "version": "1.2.1",
                            "cve_number": "CVE-1234-1234",
                            "severity": "LOW",
                            "score": "1.2",
                            "cvss_version": "2",
                            "paths": "",
                            "remarks": "NewFound",
                            "comments": "",
                        },
                        ...
                    ]
    """
    formatted_output = []
    for product_info, cve_data in all_cve_data.items():
        for cve in cve_data["cves"]:
            formatted_output.append(
                {
                    "vendor": product_info.vendor,
                    "product": product_info.product,
                    "version": product_info.version,
                    "cve_number": cve.cve_number,
                    "severity": cve.severity,
                    "score": str(cve.score),
                    "cvss_version": str(cve.cvss_version),
                    "paths": ", ".join(cve_data["paths"]),
                    "remarks": cve.remarks.name,
                    "comments": cve.comments,
                }
            )

    return formatted_output


def intermediate_output(
    all_cve_data: Dict[ProductInfo, CVEData],
    tag: str,
    scanned_dir: str,
    products_with_cve: int,
    products_without_cve: int,
    total_files: int,
) -> Dict[Dict[str, Union[str, int]], List[Dict[str, str]]]:
    """
    summary: Generate an intermediate output in the list of dictionary format with some metadata.
    Returns:
        formatted_output: Dict[Dict[str, Union[str, int]], List[Dict[str, str]]]
        - example:  {
                        metadata:   {
                                        timestamp: 2021-03-24T11:07:55Z,
                                        "tag": "backend",
                                        "scanned_dir": "/home/project/binaries",
                                        "products_with_cve": 139,
                                        "products_without_cve": 2,
                                        "total_files": 49
                                    }
                        report:     [
                                        {
                                            "vendor": "haxx"
                                            "product": "curl",
                                            "version": "1.2.1",
                                            "cve_number": "CVE-1234-1234",
                                            "severity": "LOW"
                                        },
                                        ...
                                    ]
                    }
    """

    return {
        "metadata": {
            "timestamp": datetime.now().strftime("%Y-%m-%d.%H-%M-%S"),
            "tag": tag,
            "scanned_dir": scanned_dir,
            "products_with_cve": products_with_cve,
            "products_without_cve": products_without_cve,
            "total_files": total_files,
        },
        "report": format_output(all_cve_data),
    }


def add_extension_if_not(filename: str, output_type: str) -> str:
    """
    summary: Checks if the filename ends with the extension and if not
    adds one.

    Args:
        filename (str): filename from OutputEngine
        output_type (str): contains a value from ["json", "csv", "html"]

    Returns:
        str: Filename with extension according to output_type
    """
    if not filename.endswith(f".{output_type}"):
        updated_filename = f"{filename}.{output_type}"
        return updated_filename
    else:
        return filename


def group_cve_by_remark(
    cve_by_product: List[CVE],
) -> DefaultDict[Remarks, List[Dict[str, str]]]:
    """Return a dict containing CVE details dict mapped to Remark as Key.

    Example:
    cve_by_remark = {
            "NEW":[
                {
                    "cve_number": "CVE-XXX-XXX",
                    "severity": "High",
                    "decription: "Lorem Ipsm",
                },
                {...}
            ],
            "IGNORED": [{...},{..}],
        }


    Args:
        cve_by_product (List[CVE]): List of CVE(s) that needs to be grouped

    Returns:
        DefaultDict[Remarks, List[Dict[str, str]]]: CVEs grouped by remark stored in default dict
    """
    cve_by_remarks: DefaultDict[Remarks, List[Dict[str, str]]] = defaultdict(list)
    for cve in cve_by_product:
        cve_by_remarks[cve.remarks].append(
            {
                "cve_number": cve.cve_number,
                "severity": cve.severity,
                "description": cve.description,
            }
        )
    return cve_by_remarks



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for nginx

References:
https://www.cvedetails.com/vulnerability-list/vendor_id-10048/product_id-17956/Nginx-Nginx.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=10048&product_id=17956&version_id=0&orderby=3&cvssscoremin=0

"""
from cve_bin_tool.checkers import Checker


class NginxChecker(Checker):
    CONTAINS_PATTERNS = [
        r"NGINX environment variable",
        r"nginx was built with Session Tickets support",
    ]
    FILENAME_PATTERNS = [r"nginx"]
    VERSION_PATTERNS = [r"nginx/([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("nginx", "nginx")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for icecast

https://www.cvedetails.com/product/1194/Icecast-Icecast.html?vendor_id=693

"""
from cve_bin_tool.checkers import Checker


class IcecastChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"icecast"]
    VERSION_PATTERNS = [r"Icecast ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("icecast", "icecast")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

# pylint: disable=anomalous-backslash-in-string, invalid-name
r"""
CVE checker for libexpat

References:
http://www.cvedetails.com/vulnerability-list/vendor_id-12037/product_id-22545/Libexpat-Expat.html
http://www.cvedetails.com/vulnerability-list/vendor_id-16735/product_id-39003/Libexpat-Project-Libexpat.html
https://github.com/libexpat/libexpat/blob/master/expat/Changes

RSS feeds:
http://www.cvedetails.com/vulnerability-feed.php?vendor_id=12682&product_id=0&version_id=0&orderby=3&cvssscoremin=0
http://www.cvedetails.com/vulnerability-feed.php?vendor_id=16735&product_id=0&version_id=0&orderby=3&cvssscoremin=0

Easiest way to check CVEs is currently the Changes.txt file.  You can pinpoint the CVEs using grep as follows:
grep 'Release\|CVE' Changes.txt

Which will give you output like...

Release 2.2.5 Tue October 31 2017
Release 2.2.4 Sat August 19 2017
Release 2.2.3 Wed August 2 2017
             #82  CVE-2017-11742 -- Windows: Fix DLL hijacking vulnerability
Release 2.2.2 Wed July 12 2017
Release 2.2.1 Sat June 17 2017
                  CVE-2017-9233 -- External entity infinite loop DoS
(etc.)

"""
from cve_bin_tool.checkers import Checker


class ExpatChecker(Checker):
    # FIXME: fix contains pattern
    CONTAINS_PATTERNS = [
        r"reserved prefix (xml) must not be undeclared or bound to another namespace name",
        r"cannot change setting once parsing has begun",
        "requested feature requires XML_DTD support in Expat",
    ]
    FILENAME_PATTERNS = [r"expat"]
    VERSION_PATTERNS = [r"expat_([012]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("libexpat", "expat"), ("libexpat_project", "libexpat")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
CVE-bin-tool OutputEngine tests
"""
import csv
import datetime
import json
import logging
import os
import tempfile
import unittest
from datetime import datetime

from rich.console import Console

from cve_bin_tool.output_engine import OutputEngine, output_csv, output_json
from cve_bin_tool.output_engine.console import output_console
from cve_bin_tool.output_engine.html import output_html
from cve_bin_tool.output_engine.util import format_output
from cve_bin_tool.util import CVE, CVEData, ProductInfo


class TestOutputEngine(unittest.TestCase):
    """Test the OutputEngine class functions"""

    MOCK_OUTPUT = {
        ProductInfo("vendor0", "product0", "1.0"): CVEData(
            cves=[
                CVE("CVE-1234-1234", "MEDIUM", score=4.2, cvss_version=2),
                CVE("CVE-1234-1234", "LOW", score=1.2, cvss_version=2),
            ],
            paths={""},
        ),
        ProductInfo("vendor0", "product0", "2.8.6"): CVEData(
            cves=[CVE("CVE-1234-1234", "LOW", score=2.5, cvss_version=3)], paths={""}
        ),
        ProductInfo("vendor1", "product1", "3.2.1.0"): CVEData(
            cves=[CVE("CVE-1234-1234", "HIGH", score=7.5, cvss_version=2)], paths={""}
        ),
    }

    FORMATTED_OUTPUT = [
        {
            "vendor": "vendor0",
            "product": "product0",
            "version": "1.0",
            "cve_number": "CVE-1234-1234",
            "severity": "MEDIUM",
            "score": "4.2",
            "cvss_version": "2",
            "paths": "",
            "remarks": "NewFound",
            "comments": "",
        },
        {
            "vendor": "vendor0",
            "product": "product0",
            "version": "1.0",
            "cve_number": "CVE-1234-1234",
            "severity": "LOW",
            "score": "1.2",
            "cvss_version": "2",
            "paths": "",
            "remarks": "NewFound",
            "comments": "",
        },
        {
            "vendor": "vendor0",
            "product": "product0",
            "version": "2.8.6",
            "cve_number": "CVE-1234-1234",
            "severity": "LOW",
            "score": "2.5",
            "cvss_version": "3",
            "paths": "",
            "remarks": "NewFound",
            "comments": "",
        },
        {
            "vendor": "vendor1",
            "product": "product1",
            "version": "3.2.1.0",
            "cve_number": "CVE-1234-1234",
            "severity": "HIGH",
            "score": "7.5",
            "cvss_version": "2",
            "paths": "",
            "remarks": "NewFound",
            "comments": "",
        },
    ]

    def setUp(self) -> None:
        self.output_engine = OutputEngine(
            all_cve_data=self.MOCK_OUTPUT,
            scanned_dir="",
            filename="",
            themes_dir="",
            time_of_last_update=datetime.today(),
            tag="",
        )
        self.mock_file = tempfile.NamedTemporaryFile("w+", encoding="utf-8")

    def tearDown(self) -> None:
        self.mock_file.close()

    def test_formatted_output(self):
        """Test reformatting products"""
        self.assertEqual(format_output(self.MOCK_OUTPUT), self.FORMATTED_OUTPUT)

    def test_output_json(self):
        """Test formatting output as JSON"""
        output_json(self.MOCK_OUTPUT, self.mock_file)
        self.mock_file.seek(0)  # reset file position
        self.assertEqual(json.load(self.mock_file), self.FORMATTED_OUTPUT)

    def test_output_csv(self):
        """Test formatting output as CSV"""
        output_csv(self.MOCK_OUTPUT, self.mock_file)
        self.mock_file.seek(0)  # reset file position
        reader = csv.DictReader(self.mock_file)
        expected_value = [dict(x) for x in reader]
        self.assertEqual(expected_value, self.FORMATTED_OUTPUT)

    def test_output_console(self):
        """Test Formatting Output as console"""

        console = Console(file=self.mock_file)
        output_console(
            self.MOCK_OUTPUT,
            console=console,
            time_of_last_update=datetime.today(),
        )

        expected_output = "│ vendor0 │ product0 │ 1.0     │ CVE-1234-1234 │ MEDIUM   │ 4.2 (v2)             │\n│ vendor0 │ product0 │ 1.0     │ CVE-1234-1234 │ LOW      │ 1.2 (v2)             │\n│ vendor0 │ product0 │ 2.8.6   │ CVE-1234-1234 │ LOW      │ 2.5 (v3)             │\n│ vendor1 │ product1 │ 3.2.1.0 │ CVE-1234-1234 │ HIGH     │ 7.5 (v2)             │\n└─────────┴──────────┴─────────┴───────────────┴──────────┴──────────────────────┘\n"
        self.mock_file.seek(0)  # reset file position
        result = self.mock_file.read()
        self.assertIn(expected_output, result)

    def test_output_html(self):
        """ Test formatting output as HTML """

        logger = logging.getLogger()

        output_html(
            self.MOCK_OUTPUT,
            "",
            "",
            "",
            3,
            3,
            0,
            None,
            logger,
            self.mock_file,
        )

        # sample report
        with open(
            os.path.join(os.path.dirname(__file__), "sample_report", "html_report.html")
        ) as report:
            expected_output = report.read()
            report.close()

        ## Changing some dynamic data in html reports
        # 1. date
        # 2. div IDs (dynamic ones)

        self.mock_file.seek(0)
        result = self.mock_file.read()

        result_lines = result.splitlines()

        ## date
        # date is replaced by {{date}} in the result
        # The respective change is also made in the sample report

        result = result.replace(datetime.now().strftime("%d %b %Y"), r"{{date}}")

        ## div IDs
        # div IDs, in the dashboard, which were created dynamically is being replaced by their respective line numbers
        # The respective changes are also made in the sample report

        # This signature is always a line or two before the dynamic data
        signature = '<div class="card-body">'

        for line_number in range(len(result_lines)):
            if signature in result_lines[line_number]:

                if "<!--" in result_lines[line_number + 1]:
                    # Sometimes there is a comment line after the signature line, hence it's skipped
                    line_number += 2
                else:
                    line_number += 1

                line = result_lines[line_number]  # line with the dynamic data
                index_div_id = line.index('<div id="') + len('<div id="')
                div_id = ""  # div ID of the dynamic part

                char = line[index_div_id]
                while char != '"':
                    div_id += char
                    index_div_id += 1
                    char = line[index_div_id]

                result = result.replace(
                    div_id, f"{line_number+1}"
                )  # div IDs replaced with line numbers

        assert expected_output, result

    def test_output_file(self):
        """Test file generation logic in output_file"""
        logger = logging.getLogger()

        with self.assertLogs(logger, logging.INFO) as cm:
            self.output_engine.output_file(output_type="json")

        contains_filename = False
        contains_msg = False

        filename = self.output_engine.filename

        if os.path.isfile(filename):
            contains_filename = True

        if "Output stored at" in cm.output[0]:
            contains_msg = True

        # reset everything back
        os.remove(filename)
        self.output_engine.filename = ""

        self.assertEqual(contains_filename, True)
        self.assertEqual(contains_msg, True)

    def test_output_file_filename_already_exists(self):
        """Tests output_file when filename already exist"""

        # update the filename in output_engine
        self.output_engine.filename = "testfile.csv"

        # create a file with the same name as output_engine.filename
        with open("testfile.csv", "w") as f:
            f.write("testing")

        logger = logging.getLogger()

        # setup the context manager
        with self.assertLogs(logger, logging.INFO) as cm:
            self.output_engine.output_file(output_type="csv")

        # logs to check in cm
        msg_generate_filename = (
            "Generating a new filename with Default Naming Convention"
        )
        msg_failed_to_write = "Failed to write at 'testfile.csv'. File already exists"

        # flags for logs
        contains_fail2write = False
        contains_gen_file = False

        # check if the logger contains msg
        for log in cm.output:
            if msg_generate_filename in log:
                contains_gen_file = True
            elif msg_failed_to_write in log:
                contains_fail2write = True

        # remove the generated files and reset updated variables
        os.remove("testfile.csv")
        os.remove(self.output_engine.filename)
        self.output_engine.filename = ""

        # assert
        self.assertEqual(contains_gen_file, True)
        self.assertEqual(contains_fail2write, True)

    def test_output_file_incorrect_filename(self):
        """Tests filenames that are incorrect or are not accessible"""

        # update the filename in output_engine
        self.output_engine.filename = "/not/a/good_filename"

        logger = logging.getLogger()

        # setup the context manager
        with self.assertLogs(logger, logging.INFO) as cm:
            self.output_engine.output_file(output_type="csv")

        # log to check
        msg_switch_back = "Switching Back to Default Naming Convention"

        # flags
        contains_sb = False

        for log in cm.output:
            if msg_switch_back in log:
                contains_sb = True

        # remove the generated files and reset updated variables
        os.remove(self.output_engine.filename)
        self.output_engine.filename = ""

        # assert
        self.assertEqual(contains_sb, True)



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
CVE-bin-tool file tests
"""
import os
from test.utils import event_loop

import pytest

from cve_bin_tool.async_utils import NamedTemporaryFile, aio_rmfile
from cve_bin_tool.file import aio_is_binary

ASSETS_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)), "assets")


class TestFile:
    """Tests the CVE Bin Tool file binary checker."""

    @pytest.mark.asyncio
    async def _check_test(self, file_type):
        """Helper function to parse a binary file and check whether
        the given string is in the parsed result"""
        async with NamedTemporaryFile("w+b", suffix=file_type, delete=False) as f:
            if file_type == "out":
                # write magic signature
                await f.write(b"\x7f\x45\x4c\x46\x02\x01\x01\x03\n")
                await f.seek(0)
                assert await aio_is_binary(f.name)
            else:
                await f.write(b"some other data\n")
                await f.seek(0)
                assert not await aio_is_binary(f.name)
        await aio_rmfile(f.name)

    @pytest.mark.asyncio
    async def test_binary_out_file(self):
        """file *.out"""
        await self._check_test("out")

    @pytest.mark.asyncio
    async def test_source_file(self):
        """file *.c"""
        await self._check_test("c")

    @pytest.mark.asyncio
    async def test_single_byte_file(self):
        """file single-byte"""
        assert not await aio_is_binary(os.path.join(ASSETS_PATH, "single-byte.txt"))

    @pytest.mark.asyncio
    async def test_windows(self):
        """file single-byte"""
        assert await aio_is_binary(os.path.join(ASSETS_PATH, "windows.txt"))



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "openssl",
        "version": "1.1.1d",
        "version_strings": ["part of OpenSSL 1.1.1d ", "OpenSSL 1.1.1d "],
    },
    {
        "product": "openssl",
        "version": "1.0.2g",
        "version_strings": ["part of OpenSSL 1.0.2g ", "OpenSSL 1.0.2g "],
    },
]
package_test_data = [
    {
        "url": "http://rpmfind.net/linux/mageia/distrib/5/i586/media/core/updates/",
        "package_name": "openssl-1.0.2g-1.1.mga5.i586.rpm",
        "product": "openssl",
        "version": "1.0.2g",
    },
    {
        "url": "https://files.pythonhosted.org/packages/ba/91/84a29d6a27fd6dfc21f475704c4d2053d58ed7a4033c2b0ce1b4ca4d03d9/",
        "package_name": "cryptography-3.0-cp35-abi3-manylinux2010_x86_64.whl",
        "product": "openssl",
        "version": "1.1.1g",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "berkeley_db",
        "version": "11.2.5.1.29",
        "version_strings": [
            "BDB1568 Berkeley DB library does not support DB_REGISTER on this system",
            "Berkeley DB 11g Release 2, library version 11.2.5.1.29: (date goes here)",
        ],
    }
]
package_test_data = [
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "libdb-5.3.21-25.el7.i686.rpm",
        "product": "berkeley_db",
        "version": "11.2.5.3.21",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "mariadb",
        "version": "10.3.22",
        "version_strings": [
            r"10.3.22-MariaDB",
            r"Oracle, MariaDB Corporation Ab and others.",
            r"General information about MariaDB can be found at\nhttp://mariadb.org",
            r"Welcome to the MariaDB monitor.",
            r"MariaDB virtual IO plugin for socket communication",
        ],
    }
]
package_test_data = [
    {
        "url": "http://ports.ubuntu.com/pool/universe/m/mariadb-10.3/",
        "package_name": "mariadb-client-core-10.3_10.3.22-1ubuntu1_arm64.deb",
        "product": "mariadb",
        "version": "10.3.22",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "openssh", "version": "6.9", "version_strings": ["OpenSSH_6.9"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/openssh/6.8p1/1.1.fc23/x86_64/",
        "package_name": "openssh-clients-6.8p1-1.1.fc23.x86_64.rpm",
        "product": "openssh",
        "version": "6.8p1",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

# pylint: disable=invalid-name
"""
CVE checker for libgcrypt

https://www.cvedetails.com/product/25777/Gnupg-Libgcrypt.html?vendor_id=4711
"""
from cve_bin_tool.checkers import Checker


class LibgcryptChecker(Checker):
    CONTAINS_PATTERNS = [
        "fork detection failed",
        "too many random bits requested",
        "severe error getting random",
    ]
    FILENAME_PATTERNS = [r"libgcrypt.so."]
    VERSION_PATTERNS = [r"Libgcrypt ([01]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("gnupg", "libgcrypt")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "irssi", "version": "1.0.0", "version_strings": ["irssi 1.0.0"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/irssi/1.0.1/1.fc26/x86_64/",
        "package_name": "irssi-1.0.1-1.fc26.x86_64.rpm",
        "product": "irssi",
        "version": "1.0.1",
    },
    {
        "url": "https://mirrors.kernel.org/mageia/distrib/5/x86_64/media/core/release/",
        "package_name": "irssi-0.8.16-4.mga5.x86_64.rpm",
        "product": "irssi",
        "version": "0.8.16",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for libsndfile

https://www.cvedetails.com/product/36889/Libsndfile-Project-Libsndfile.html?vendor_id=16294

"""
from cve_bin_tool.checkers import Checker


class LibsndfileChecker(Checker):
    CONTAINS_PATTERNS = [
        r"This version of libsndfile was compiled without Ogg/Speex support.",
        r"No error defined for this error number. This is a bug in libsndfile.",
        r"NULL SF_INFO pointer passed to libsndfile.",
        r"MATLAB 5.0 MAT-file, written by libsndfile-\d+\.\d+\.\d+,",
    ]
    FILENAME_PATTERNS = [r"libsndfile.so"]
    VERSION_PATTERNS = [r"libsndfile-(\d+\.\d+\.\d+)"]
    VENDOR_PRODUCT = [("libsndfile_project", "libsndfile")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "libxml2",
        "version": "2.9.0",
        "version_strings": ["/libxml2-2.9.0/", "xmlNewElementContent : name != NULL !"],
    }
]
package_test_data = [
    {
        "url": "https://archives.fedoraproject.org/pub/archive/fedora/linux/releases/30/Everything/x86_64/os/Packages/l/",
        "package_name": "libxml2-2.9.9-2.fc30.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.9",
    },
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "libxml2-2.9.1-6.el7.5.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.1",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.7.2-1.x86_64.rpm",
        "product": "libxml2",
        "version": "2.7.2",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.7.3-1.x86_64.rpm",
        "product": "libxml2",
        "version": "2.7.3",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.7.4-1.x86_64.rpm",
        "product": "libxml2",
        "version": "2.7.4",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.7.5-1.x86_64.rpm",
        "product": "libxml2",
        "version": "2.7.5",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.7.6-1.x86_64.rpm",
        "product": "libxml2",
        "version": "2.7.6",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.7.7-1.x86_64.rpm",
        "product": "libxml2",
        "version": "2.7.7",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.7.8-1.x86_64.rpm",
        "product": "libxml2",
        "version": "2.7.8",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.8.0-1.x86_64.rpm",
        "product": "libxml2",
        "version": "2.8.0",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.0-0rc0.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.0",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.0-0rc1.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.0",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.0-0rc2.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.0",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.0-1.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.0",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.1-1.fc17.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.1",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.2-0rc1.fc19.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.2",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.2-0rc2.fc19.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.2",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.2-1.fc19.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.2",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.3-1.fc23.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.3",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.4-0rc1.fc23.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.4",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.4-0rc2.fc23.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.4",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.4-1.fc23.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.4",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.5-0rc1.fc24.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.5",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.5-0rc2.fc24.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.5",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.5-1.fc24.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.5",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.6-0rc1.fc26.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.6",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.6-1.fc26.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.6",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.7-0rc1.fc26.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.7",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.7-1.fc26.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.7",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.8-0rc1.fc26.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.8",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.8-1.fc26.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.8",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.9-0rc1.fc28.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.9",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.9-0rc2.fc28.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.9",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxml2-2.9.9-1.fc28.x86_64.rpm",
        "product": "libxml2",
        "version": "2.9.9",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

# pylint: disable=invalid-name

"""
CVE checker for libxerces

References:
http://www.cvedetails.com/vulnerability-list/vendor_id-45/product_id-4103/Apache-Xerces-c-.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=45&product_id=4103&version_id=&orderby=2&cvssscoremin=0
"""
from cve_bin_tool.checkers import Checker


class XercesChecker(Checker):
    CONTAINS_PATTERNS = [r"xerces-c-src_"]
    FILENAME_PATTERNS = [r"libxerces-c.so", r"libxerces-c-3.1.so"]
    VERSION_PATTERNS = [
        r"\/xerces-c-src_([0-9]+_[0-9]+_[0-9]+)\/",
        r"xercesc_([0-9]+\_[0-9]+):",
    ]
    VENDOR_PRODUCT = [("apache", "xerces-c")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "avahi", "version": "0.6.24", "version_strings": ["avahi 0.6.24"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/avahi/0.6.26/3.fc14/x86_64/",
        "package_name": "avahi-0.6.26-3.fc14.x86_64.rpm",
        "product": "avahi",
        "version": "0.6.26",
    },
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "avahi-0.6.31-20.el7.x86_64.rpm",
        "product": "avahi",
        "version": "0.6.31",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "p7zip",
        "version": "16.02",
        "version_strings": ["7-Zip [64] 16.02 : Copyright"],
    }
]
package_test_data = [
    {
        "url": "https://download-ib01.fedoraproject.org/pub/fedora/linux/releases/33/Everything/aarch64/os/Packages/p/",
        "package_name": "p7zip-16.02-18.fc33.aarch64.rpm",
        "product": "p7zip",
        "version": "16.02",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "nginx",
        "version": "1.13.2",
        "version_strings": ["NGINX environment variable", "nginx/1.13.2"],
    }
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/nginx/1.8.0/10.fc22/x86_64/",
        "package_name": "nginx-1.8.0-10.fc22.x86_64.rpm",
        "product": "nginx",
        "version": "1.8.0",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for libxslt
https://www.cvedetails.com/product/14676/Xmlsoft-Libxslt.html?vendor_id=1962
"""
from cve_bin_tool.checkers import Checker


class LibxsltChecker(Checker):
    CONTAINS_PATTERNS = [
        r"libxslt.so.1",
        r"xsltLibxsltVersion",
        r"Using libxml %s, libxslt %s and libexslt %s",
        r"xsltproc was compiled against libxml %d, libxslt %d and libexslt %d",
        r"libxslt %d was compiled against libxml %d",
    ]
    FILENAME_PATTERNS = [r"xsltproc"]
    VERSION_PATTERNS = [r"libxslt\.so\.([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("xmlsoft", "libxslt")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "dpkg",
        "version": "1.20.9",
        "version_strings": [
            "dpkg-1.20.9",
            "dpkg-deb-1.20.9",
            "dpkg-query-1.20.9",
            "dpkg-divert-1.20.9",
        ],
    },
    {
        "product": "dpkg",
        "version": "1.18.25",
        "version_strings": [
            "dpkg-1.18.25",
            "dpkg-deb-1.18.25",
            "dpkg-query-1.18.25",
            "dpkg-divert-1.18.25",
        ],
    },
]
package_test_data = [
    {
        "url": "https://download-ib01.fedoraproject.org/pub/fedora/linux/development/rawhide/Everything/aarch64/os/Packages/d/",
        "package_name": "dpkg-1.20.9-2.fc35.aarch64.rpm",
        "product": "dpkg",
        "version": "1.20.9",
    },
    {
        "url": "https://download-ib01.fedoraproject.org/pub/epel/8/Everything/aarch64/Packages/d/",
        "package_name": "dpkg-1.18.25-12.el8.aarch64.rpm",
        "product": "dpkg",
        "version": "1.18.25",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import csv
import re
import subprocess
from importlib.metadata import version
from os.path import dirname, join

ROOT_PATH = join(dirname(__file__), "..")

REQ_TXT = join(ROOT_PATH, "requirements.txt")
REQ_CSV = join(ROOT_PATH, "requirements.csv")
DOC_TXT = join(ROOT_PATH, "doc", "requirements.txt")
DOC_CSV = join(ROOT_PATH, "doc", "requirements.csv")

SCAN_CSV = join(ROOT_PATH, "cve_bin_tool_requirements.csv")

HTML_DEP_PATH = join(
    ROOT_PATH,
    "cve_bin_tool",
    "output_engine",
    "html_reports",
    "js",
)

HTML_DEP_CSV = join(HTML_DEP_PATH, "dependencies.csv")

# Dependencies that currently have CVEs
# Remove from the list once they are updated
ALLOWED_PACKAGES = ["reportlab"]


def get_out_of_sync_packages(csv_name, txt_name):

    new_packages = set()
    removed_packages = set()
    csv_package_names = set()
    txt_package_names = set()

    with open(csv_name) as csv_file, open(txt_name) as txt_file:
        csv_reader = csv.reader(csv_file)
        next(csv_reader)
        for (_, product) in csv_reader:
            csv_package_names.add(product)
        lines = txt_file.readlines()
        for line in lines:
            txt_package_names.add(re.split(">|\\[|;|=|\n", line)[0])
        new_packages = txt_package_names - csv_package_names
        removed_packages = csv_package_names - txt_package_names

    return (new_packages, removed_packages)


# Test to check if the requirements.csv files are in sync with requirements.txt files
def test_txt_csv_sync():

    errors = set()

    (
        req_new_packages,
        req_removed_packages,
    ) = get_out_of_sync_packages(REQ_CSV, REQ_TXT)
    (
        doc_new_packages,
        doc_removed_packages,
    ) = get_out_of_sync_packages(DOC_CSV, DOC_TXT)

    if doc_removed_packages != set():
        errors.add(
            f"The requirements.txt and requirements.csv files of docs are out of sync! Please remove {', '.join(doc_removed_packages)} from the respective requirements.csv file\n"
        )
    if doc_new_packages != set():
        errors.add(
            f"The requirements.txt and requirements.csv files of docs are out of sync! Please add {', '.join(doc_new_packages)} to the respective requirements.csv file\n"
        )
    if req_removed_packages != set():
        errors.add(
            f"The requirements.txt and requirements.csv files of cve-bin-tool are out of sync! Please remove {', '.join(req_removed_packages)} from the respective requirements.csv file\n"
        )
    if req_new_packages != set():
        errors.add(
            f"The requirements.txt and requirements.csv files of cve-bin-tool are out of sync! Please add {', '.join(req_new_packages)} to the respective requirements.csv file\n"
        )

    assert errors == set(), f"The error(s) are:\n {''.join(errors)}"


def get_cache_csv_data(file):

    data = []

    with open(file) as f:
        r = csv.reader(f)
        next(r)
        for (vendor, product) in r:
            if file is HTML_DEP_CSV:
                file_name = f"{HTML_DEP_PATH}/{product}"
                if not file_name.endswith(".js"):
                    file_name += ".js"
                with open(file_name) as f:
                    file_content = f.read()
                    html_dep_version = re.search(
                        r"v([0-9]+\.[0-9]+\.[0-9]+)", file_content
                    ).group(1)
                    if product not in ALLOWED_PACKAGES:
                        data.append((vendor, product, html_dep_version))
            else:
                if "_not_in_db" not in vendor and product not in ALLOWED_PACKAGES:
                    data.append((vendor, product, version(product)))

        return data


# Test to check for CVEs in cve-bin-tool requirements/dependencies
def test_requirements():

    cache_csv_data = (
        get_cache_csv_data(REQ_CSV)
        + get_cache_csv_data(DOC_CSV)
        + get_cache_csv_data(HTML_DEP_CSV)
    )

    # writes a cache CSV file
    with open(SCAN_CSV, "w") as f:
        writer = csv.writer(f)
        fieldnames = ["vendor", "product", "version"]
        writer = csv.writer(f)
        writer.writerow(fieldnames)
        for row in cache_csv_data:
            writer.writerow(row)

    cve_check = subprocess.run(
        ["python", "-m", "cve_bin_tool.cli", "--input-file", SCAN_CSV]
    )
    assert (
        cve_check.returncode == 0
    ), f"{cve_check.returncode} dependencies/requirements have CVEs"



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

from re import UNICODE

import pytest

from cve_bin_tool.error_handler import UnknownArchiveType
from cve_bin_tool.helper_script import HelperScript


class TestHelperScript:
    @pytest.mark.parametrize(
        "filename, product_name",
        [
            (
                "libsndfile-1.0.25-12.el7.x86_64.rpm",
                "libsndfile",
            ),  # normal packages (with 2 '-')
            (
                "libsndfile-1.0.31-1-aarch64.pkg.tar.xz",
                "libsndfile",
            ),  # .tar.xz packages (with 3 '-')
            (
                "logrotate_3.14.0-4ubuntu5_amd64.deb",
                "logrotate",
            ),  # .deb packages (with 2 '_')
            (
                "libarchive_3.4.2-1_aarch64_cortex-a72.ipk",
                "libarchive",
            ),  # .ipk packages (with 3 '_')
            (
                "tomcat9_9.0.37-3_all.deb",
                "tomcat9",
            ),  # packages with digits in product_name
            (
                "openssh-client_8.4p1-5ubuntu1_amd64.deb",
                "openssh-client",
            ),  # packages with '-' in product_name
            (
                "./Packages/bash-4.2.46-34.el7.x86_64.rpm",
                "bash",
            ),  # directaory names
        ],
    )
    def test_parse_filename(self, filename, product_name):
        hs = HelperScript(filename)
        assert hs.parse_filename(filename) == product_name

    # @pytest.mark.parametrize("filename", [
    #     "bash-4.2.46-34.el7.x86_64.abc" # unsupported file type
    # ])
    # def test_parse_filename(self, filename):
    #     hs = HelperScript(filename)
    #     with pytest.raises(UnknownArchiveType):
    #         hs.parse_filename(filename)



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import os
import subprocess
import sys
from re import search

import pkg_resources

from cve_bin_tool.egg_updater import IS_DEVELOP, update_egg
from cve_bin_tool.error_handler import ErrorMode
from cve_bin_tool.extractor import Extractor
from cve_bin_tool.file import is_binary
from cve_bin_tool.log import LOGGER
from cve_bin_tool.package_list_parser.vendor_fetch import VendorFetch
from cve_bin_tool.strings import Strings
from cve_bin_tool.util import DirWalk, ProductInfo, inpath


class InvalidFileError(Exception):
    """Filepath is invalid for scanning."""


class VersionScanner:
    """ "Scans files for CVEs using CVE checkers"""

    CHECKER_ENTRYPOINT = "cve_bin_tool.checker"

    def __init__(
        self,
        should_extract=False,
        exclude_folders=[],
        checkers=None,
        logger=None,
        error_mode=ErrorMode.TruncTrace,
        score=0,
    ):
        self.logger = logger or LOGGER.getChild(self.__class__.__name__)
        # Update egg if installed in development mode
        if IS_DEVELOP():
            self.logger.info("Updating egg_info")
            update_egg()

        # Load checkers if not given
        self.checkers = checkers or self.load_checkers()
        self.score = score
        self.total_scanned_files = 0
        self.exclude_folders = exclude_folders + [".git"]

        self.walker = DirWalk(
            folder_exclude_pattern=";".join(
                exclude if exclude.endswith("*") else exclude + "*"
                for exclude in exclude_folders
            )
        ).walk
        self.should_extract = should_extract
        self.file_stack = []
        self.error_mode = error_mode
        # self.logger.info("Checkers loaded: %s" % (", ".join(self.checkers.keys())))

    @classmethod
    def load_checkers(cls):
        """Loads CVE checkers"""
        checkers = dict(
            map(
                lambda checker: (checker.name, checker.load()),
                pkg_resources.iter_entry_points(cls.CHECKER_ENTRYPOINT),
            )
        )
        return checkers

    @classmethod
    def available_checkers(cls):
        checkers = pkg_resources.iter_entry_points(cls.CHECKER_ENTRYPOINT)
        checker_list = [item.name for item in checkers]
        return checker_list

    def remove_skiplist(self, skips):
        # Take out any checkers that are on the skip list
        # (string of comma-delimited checker names)
        skiplist = skips
        for skipme in skiplist:
            if skipme in self.checkers:
                del self.checkers[skipme]
                self.logger.debug(f"Skipping checker: {skipme}")
            else:
                self.logger.error(f"Checker {skipme} is not a valid checker name")

    def print_checkers(self):
        self.logger.info(f'Checkers: {", ".join(self.checkers.keys())}')

    def is_executable(self, filename):
        """check if file is an ELF binary file"""

        if inpath("file"):
            # use system file if available (for performance reasons)
            output = subprocess.check_output(["file", filename])
            output = output.decode(sys.stdout.encoding)

            if "cannot open" in output:
                self.logger.warning(f"Unopenable file {filename} cannot be scanned")
                return False

            if (
                ("LSB " not in output)
                and ("LSB shared" not in output)
                and ("LSB executable" not in output)
                and ("PE32 executable" not in output)
                and ("PE32+ executable" not in output)
                and ("Mach-O" not in output)
                and ("PKG-INFO: " not in output)
                and ("METADATA: " not in output)
            ):
                return False
        # otherwise use python implementation of file
        elif not is_binary(filename):
            return False

        return True, output

    def parse_strings(self, filename):
        """parse binary file's strings"""

        if inpath("strings"):
            # use "strings" on system if available (for performance)
            lines = (
                subprocess.check_output(["strings", filename])
                .decode("utf-8")
                .splitlines()
            )
        else:
            # Otherwise, use python implementation
            s = Strings(filename)
            lines = s.parse()
        return lines

    def scan_file(self, filename):
        """Scans a file to see if it contains any of the target libraries,
        and whether any of those contain CVEs"""

        self.logger.debug(f"Scanning file: {filename}")
        self.total_scanned_files += 1

        # Do not try to scan symlinks
        if os.path.islink(filename):
            return None

        # Ensure filename is a file
        if not os.path.isfile(filename):
            self.logger.warning(f"Invalid file {filename} cannot be scanned")
            return None

        # check if it's an ELF binary file
        try:
            t, output = self.is_executable(filename)
        except:
            t = self.is_executable(filename)

        if not t:
            return None

        # parse binary file's strings
        lines = self.parse_strings(filename)

        #  If python package then strip the lines to avoid detecting other product strings
        if "PKG-INFO: " in output or "METADATA: " in output:
            lines = lines[1:3]
            lines[0] = (
                "--generated pattern for cve-bin-tool " + lines[0] + " " + lines[1]
            )
            yield from self.run_python_package_checkers(filename, lines)

        yield from self.run_checkers(filename, lines)

    def run_python_package_checkers(self, filename, lines):
        """
        This function runs only for python packages.
        There are no actual checkers.
        The ProductInfo is computed without the help of any checkers from PKG-INFO or METADATA.
        """
        product = search(
            r"--generated pattern for cve-bin-tool Name: (.+?) Version:", lines[0]
        ).group(1)
        version = search(r"Version: (.+?)$", lines[1]).group(1)

        with VendorFetch() as vendor_fetch:
            vendor_package_pair = vendor_fetch.get_vendor_product_pairs(product)

        if vendor_package_pair != []:
            vendor = vendor_package_pair[0]["vendor"]
            file_path = "".join(self.file_stack)

            self.logger.info(f"{file_path} is {product} {version}")

            yield ProductInfo(vendor, product, version), file_path

        self.logger.debug(f"Done scanning file: {filename}")

    def run_checkers(self, filename, lines):
        # tko
        for (dummy_checker_name, checker) in self.checkers.items():
            checker = checker()
            result = checker.get_version(lines, filename)
            # do some magic so we can iterate over all results, even the ones that just return 1 hit
            if "is_or_contains" in result:
                results = [dict()]
                results[0] = result
            else:
                results = result

            for result in results:
                if "is_or_contains" in result:
                    version = "UNKNOWN"
                    if "version" in result and result["version"] != "UNKNOWN":
                        version = result["version"]
                    elif result["version"] == "UNKNOWN":
                        file_path = "".join(self.file_stack)
                        self.logger.warning(
                            f"{dummy_checker_name} was detected with version UNKNOWN in file {file_path}"
                        )
                    else:
                        self.logger.error(f"No version info for {dummy_checker_name}")

                    if version != "UNKNOWN":
                        file_path = "".join(self.file_stack)
                        self.logger.info(
                            f'{file_path} {result["is_or_contains"]} {dummy_checker_name} {version}'
                        )
                        for vendor, product in checker.VENDOR_PRODUCT:
                            yield ProductInfo(vendor, product, version), file_path

        self.logger.debug(f"Done scanning file: {filename}")

    @staticmethod
    def clean_file_path(filepath):
        """Returns a cleaner filepath by removing temp path from filepath"""

        # we'll recieve a filepath similar to
        # /temp/anything/extractable_filename.extracted/folders/inside/file
        # We'll return /folders/inside/file to be scanned

        # start_point is the point from we want to start trimming
        # len("extracted") = 9
        start_point = filepath.find("extracted") + 9
        return filepath[start_point:]

    def scan_and_or_extract_file(self, ectx, filepath):
        """Runs extraction if possible and desired otherwise scans."""
        # Scan the file
        yield from self.scan_file(filepath)
        # Attempt to extract the file and scan the contents
        if ectx.can_extract(filepath):
            if not self.should_extract:
                LOGGER.warning(
                    f"{filepath} is an archive. Pass -x option to auto-extract"
                )
                return None
            for filename in self.walker([ectx.extract(filepath)]):
                clean_path = self.clean_file_path(filename)
                self.file_stack.append(f" contains {clean_path}")
                yield from self.scan_and_or_extract_file(ectx, filename)
                self.file_stack.pop()

    def recursive_scan(self, scan_path):
        with Extractor(logger=self.logger, error_mode=self.error_mode) as ectx:
            if os.path.isdir(scan_path):
                for filepath in self.walker([scan_path]):
                    self.file_stack.append(filepath)
                    yield from self.scan_and_or_extract_file(ectx, filepath)
                    self.file_stack.pop()
            elif os.path.isfile(scan_path):
                self.file_stack.append(scan_path)
                yield from self.scan_and_or_extract_file(ectx, scan_path)
                self.file_stack.pop()



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "mysql",
        "version": "8.0.20",
        "version_strings": [
            r"mysql-8.0-8.0.20",
            r"To buy MySQL Enterprise support, training, or other products, visit:",
        ],
    }
]
package_test_data = [
    {
        "url": "http://ports.ubuntu.com/pool/main/m/mysql-8.0/",
        "package_name": "mysql-client-core-8.0_8.0.19-0ubuntu5_arm64.deb",
        "product": "mysql",
        "version": "8.0.19",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for rsyslog
https://www.cvedetails.com/product/15708/Rsyslog-Rsyslog.html?vendor_id=3361
"""
from cve_bin_tool.checkers import Checker


class RsyslogChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"rsyslogd"]
    VERSION_PATTERNS = [r"rsyslog ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("rsyslog", "rsyslog")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "syslog-ng",
        "version": "1.5.15",
        "version_strings": ["syslog-ng-1.5.15", "Set syslog-ng control socket"],
    }
]
package_test_data = [
    {
        "url": "https://mirrors.kernel.org/fedora-buffet/archive/fedora/linux/releases/21/Everything/x86_64/os/Packages/s/",
        "package_name": "syslog-ng-3.5.6-3.fc21.x86_64.rpm",
        "product": "syslog-ng",
        "version": "3.5.6",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for openvpn

https://www.cvedetails.com/product/5768/Openvpn-Openvpn.html?vendor_id=3278

"""
from cve_bin_tool.checkers import Checker


class OpenvpnChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"openvpn"]
    VERSION_PATTERNS = [r"OpenVPN ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("openvpn", "openvpn")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import os
import sqlite3
import sys
from collections import defaultdict
from logging import Logger
from string import ascii_lowercase
from typing import DefaultDict, Dict, List

from pkg_resources import parse_version
from rich.columns import Columns
from rich.console import Console
from rich.panel import Panel

from cve_bin_tool.cvedb import DBNAME, DISK_LOCATION_DEFAULT
from cve_bin_tool.error_handler import ErrorMode
from cve_bin_tool.input_engine import TriageData
from cve_bin_tool.linkify import linkify_cve
from cve_bin_tool.log import LOGGER
from cve_bin_tool.theme import cve_theme
from cve_bin_tool.util import CVE, CVEData, ProductInfo


class CVEScanner:
    """
    This class is for reading CVEs from the database
    """

    products_with_cve: int
    products_without_cve: int
    all_cve_data: DefaultDict[ProductInfo, CVEData]

    RANGE_UNSET: str = ""
    dbname: str = os.path.join(DISK_LOCATION_DEFAULT, DBNAME)
    CONSOLE: Console = Console(file=sys.stderr, theme=cve_theme)
    ALPHA_TO_NUM: Dict[str, int] = dict(zip(ascii_lowercase, range(26)))

    def __init__(
        self,
        score: int = 0,
        logger: Logger = None,
        error_mode: ErrorMode = ErrorMode.TruncTrace,
    ):
        self.logger = logger or LOGGER.getChild(self.__class__.__name__)
        self.error_mode = error_mode
        self.score = score
        self.products_with_cve = 0
        self.products_without_cve = 0
        self.all_cve_data = defaultdict(CVEData)

    def get_cves(self, product_info: ProductInfo, triage_data: TriageData):
        """Get CVEs against a specific version of a product.

        Example:
            nvd.get_cves('haxx', 'curl', '7.34.0')
        """
        if product_info in self.all_cve_data:
            # If product_info already in all_cve_data no need to fetch cves from database again
            # We just need to update paths.
            self.products_with_cve += 1
            self.all_cve_data[product_info]["paths"] |= triage_data["paths"]
            return

        # Check for anything directly marked
        query = """
        SELECT CVE_number FROM cve_range
        WHERE vendor=? AND product=? AND version=?
        """
        # Removing * from vendors that are guessed by the package list parser
        vendor = product_info.vendor.replace("*", "")
        self.cursor.execute(query, [vendor, product_info.product, product_info.version])

        cve_list = list(map(lambda x: x[0], self.cursor.fetchall()))

        # Check for any ranges
        query = """
        SELECT
            CVE_number,
            versionStartIncluding,
            versionStartExcluding,
            versionEndIncluding,
            versionEndExcluding
        FROM cve_range
        WHERE vendor=? AND product=? AND version=?
        """

        # Removing * from vendors that are guessed by the package list parser
        vendor = product_info.vendor.replace("*", "")
        self.cursor.execute(query, [vendor, product_info.product, "*"])

        for cve_range in self.cursor:
            (
                cve_number,
                versionStartIncluding,
                versionStartExcluding,
                versionEndIncluding,
                versionEndExcluding,
            ) = cve_range

            parsed_version = parse_version(product_info.version)

            # pep-440 doesn't include versions of the type 1.1.0g used by openssl
            # so if this is openssl, convert the last letter to a .number
            if product_info.product == "openssl":
                # if last character is a letter, convert it to .number
                # version = self.openssl_convert(product_info.version)
                versionStartIncluding = self.openssl_convert(versionStartIncluding)
                versionStartExcluding = self.openssl_convert(versionStartExcluding)
                versionEndIncluding = self.openssl_convert(versionEndIncluding)
                versionEndExcluding = self.openssl_convert(versionEndExcluding)
                parsed_version = parse_version(
                    self.openssl_convert(product_info.version)
                )

            # check the start range
            passes_start = False
            if (
                versionStartIncluding is not self.RANGE_UNSET
                and parsed_version >= parse_version(versionStartIncluding)
            ):
                passes_start = True
            if (
                versionStartExcluding is not self.RANGE_UNSET
                and parsed_version > parse_version(versionStartExcluding)
            ):
                passes_start = True

            if (
                versionStartIncluding is self.RANGE_UNSET
                and versionStartExcluding is self.RANGE_UNSET
            ):
                # then there is no start range so just say true
                passes_start = True

            # check the end range
            passes_end = False
            if (
                versionEndIncluding is not self.RANGE_UNSET
                and parsed_version <= parse_version(versionEndIncluding)
            ):
                passes_end = True

            if (
                versionEndExcluding is not self.RANGE_UNSET
                and parsed_version < parse_version(versionEndExcluding)
            ):
                passes_end = True
            if (
                versionEndIncluding is self.RANGE_UNSET
                and versionEndExcluding is self.RANGE_UNSET
            ):
                # then there is no end range so it passes
                passes_end = True
            # if it fits into both ends of the range, add the cve number
            if passes_start and passes_end:
                cve_list.append(cve_number)

        # Go through and get all the severities
        if cve_list:
            query = f"""
            SELECT CVE_number, severity, description, score, cvss_version
            FROM cve_severity
            WHERE CVE_number IN ({",".join(["?"] * len(cve_list))}) AND score >= ?
            ORDER BY CVE_number
            """
            # Add score parameter to tuple listing CVEs to pass to query
            cve_list.append(self.score)

            result = self.cursor.execute(query, cve_list)

            cves: List[CVE] = []
            for row in result:
                triage = triage_data.get(row["cve_number"]) or triage_data.get(
                    "default"
                )
                # Only scan cves if triage is not None.
                # Triage will only be None if triage_data don't have default attribute.
                # NOTE: Triage can be empty dictionary so checking `if triage:` won't suffice.
                if triage is not None:
                    row_dict = dict(row)
                    row_dict.update(triage)
                    # print(row_dict)
                    row_dict["severity"] = row_dict["severity"] or row["severity"]
                    row_dict["score"] = row_dict["score"] or row["score"]
                    row_dict["cvss_version"] = (
                        row_dict["cvss_version"] or row["cvss_version"]
                    )
                    cve = CVE(**row_dict)
                    cves.append(cve)

            if cves:
                self.products_with_cve += 1
                self.logger.info(f"Known CVEs in {product_info}")
                # error_mode.value will only be greater than 1 if quiet mode.
                if self.error_mode.value > 1:
                    title = f"[default][b]{len(cves)}[/b] CVE(s) in [b]{product_info.vendor}.{product_info.product}[/b] v[b]{product_info.version}[/b]"
                    self.CONSOLE.log()
                    self.CONSOLE.log(
                        Panel(
                            Columns(
                                (
                                    linkify_cve(
                                        f"[{cve.severity.lower()}]{cve.cve_number}"
                                    )
                                    for cve in cves
                                ),
                                padding=(0, 2),
                            ),
                            padding=1,
                            border_style="red",
                            title=title,
                        )
                    )
                    self.CONSOLE.log()
            else:
                # No cves found for (product, vendor, version) tuple in the NVD database.
                self.products_without_cve += 1

            self.all_cve_data[product_info]["cves"] = cves
            self.all_cve_data[product_info]["paths"] |= triage_data["paths"]

        else:
            # There isn't any entry for (product, vendor, version) tuple in the NVD database.
            self.products_without_cve += 1
            self.logger.debug(
                f"No CVEs found for {product_info}. Is the vendor/product info correct?"
            )
            self.all_cve_data[product_info]["cves"] = [CVE("UNKNOWN", "UNKNOWN")]

    def openssl_convert(self, version: str) -> str:
        """pkg_resources follows pep-440 which doesn't expect openssl style 1.1.0g version numbering
        So to fake it, if the last character is a letter, replace it with .number before comparing"""
        if not version:  # if version is empty return it.
            return version

        last_char = version[-1]
        second_last_char = version[-2]

        if last_char in self.ALPHA_TO_NUM and second_last_char in self.ALPHA_TO_NUM:
            version = f"{version[:-2]}.{self.ALPHA_TO_NUM[second_last_char]}.{self.ALPHA_TO_NUM[last_char]}"

        elif last_char in self.ALPHA_TO_NUM:
            version = f"{version[:-1]}.{self.ALPHA_TO_NUM[last_char]}"
        return version

    def affected(self):
        """Returns list of product name and version tuples identified from
        scan"""
        return sorted(
            (cve_data.product, cve_data.version) for cve_data in self.all_cve_data
        )

    def __enter__(self):
        self.connection = sqlite3.connect(self.dbname)
        self.connection.row_factory = sqlite3.Row
        self.cursor = self.connection.cursor()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.cursor.close()
        self.connection.close()



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for libarchive

https://www.cvedetails.com/product/26168/Libarchive-Libarchive.html?vendor_id=12872

"""
from cve_bin_tool.checkers import Checker


class LibarchiveChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"libarchive.so"]
    VERSION_PATTERNS = [r"libarchive ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("libarchive", "libarchive")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "gpgme",
        "version": "1.16.0",
        "version_strings": [
            "GPGME-Tool 1.16.0 ready",
            "This is GPGME 1.16.0 - The GnuPG Made Easy library",
        ],
    },
    {
        "product": "gpgme",
        "version": "1.13.1",
        "version_strings": [
            "GPGME-Tool 1.13.1 ready",
            "This is GPGME 1.13.1 - The GnuPG Made Easy library",
        ],
    },
]
package_test_data = [
    {
        "url": "https://eu.mirror.archlinuxarm.org/aarch64/core/",
        "package_name": "gpgme-1.16.0-1-aarch64.pkg.tar.xz",
        "product": "gpgme",
        "version": "1.16.0",
    },
    {
        "url": "https://ftp.lysator.liu.se/pub/opensuse/ports/aarch64/distribution/leap/15.2/repo/oss/aarch64/",
        "package_name": "gpgme-1.13.1-lp152.2.1.aarch64.rpm",
        "product": "gpgme",
        "version": "1.13.1",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
CVE checker for GNUPG

https://www.cvedetails.com/vulnerability-list/vendor_id-4711/product_id-8075/Gnupg-Gnupg.html

"""
from cve_bin_tool.checkers import Checker


class GnupgChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [
        "gpg",
        "g13",
    ]
    VERSION_PATTERNS = [
        r"GNU Privacy Guard's OpenPGP server ([0-9]+\.[0-9]+\.[0-9]+) ready",
        r"GNU Privacy Guard's G13 server ([0-9]+\.[0-9]+\.[0-9]+) ready",
    ]
    VENDOR_PRODUCT = [("gnupg", "gnupg")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for libtiff

http://www.cvedetails.com/vulnerability-list/vendor_id-2224/product_id-3881/Libtiff-Libtiff.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=2224&product_id=3881&version_id=&orderby=3&cvssscoremin=0
"""
from cve_bin_tool.checkers import Checker


class LibtiffChecker(Checker):
    CONTAINS_PATTERNS = [
        r"LIBTIFF, Version ",
        r"Unknown TIFF resolution unit %d ignored",
        r'TIFF directory is missing required "StripByteCounts" field, calculating from imagelength',
    ]
    FILENAME_PATTERNS = [r"libtiff.so."]
    VERSION_PATTERNS = [r"LIBTIFF, Version ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("libtiff", "libtiff")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for strongswan

https://www.cvedetails.com/product/3992/Strongswan-Strongswan.html?vendor_id=2278

"""
from cve_bin_tool.checkers import Checker


class StrongswanChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"libcharon.so"]
    VERSION_PATTERNS = [r"strongSwan ([0-9]+\.[0-9]+\.[0-9])"]
    VENDOR_PRODUCT = [("strongswan", "strongswan")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "gnupg",
        "version": "2.2.27",
        "version_strings": ["GNU Privacy Guard's OpenPGP server 2.2.27 ready"],
    },
    {
        "product": "gnupg",
        "version": "2.2.23",
        "version_strings": ["GNU Privacy Guard's G13 server 2.2.23 ready"],
    },
]

package_test_data = [
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "gnupg2-2.0.22-5.el7_5.x86_64.rpm",
        "product": "gnupg",
        "version": "2.0.22",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "systemd",
        "version": "239",
        "version_strings": ["LIBSYSTEMD_239", "sd_bus_error_copy"],
    }
]
package_test_data = [
    {
        "url": "http://mirror.centos.org/centos/8/BaseOS/x86_64/os/Packages/",
        "package_name": "systemd-239-40.el8.i686.rpm",
        "product": "systemd",
        "version": "239",
    },
    {
        "url": "http://security.ubuntu.com/ubuntu/pool/main/s/systemd/",
        "package_name": "systemd_229-4ubuntu21.27_amd64.deb",
        "product": "systemd",
        "version": "229",
    },
    {
        "url": "https://rpmfind.net/linux/openmandriva/4.1/repository/x86_64/main/release/",
        "package_name": "systemd-244.20191203-2-omv4001.x86_64.rpm",
        "product": "systemd",
        "version": "244",
    },
    # {
    #    "url": "https://rpmfind.net/linux/fedora/linux/releases/33/Everything/x86_64/os/Packages/s/",
    #    "package_name": "systemd-246.6-3.fc33.i686.rpm",
    #    "product": "systemd",
    #    "version": "243",
    # },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "curl", "version": "7.34.0", "version_strings": ["curl 7.34.0"]}
]
package_test_data = [
    {
        "url": "https://archives.fedoraproject.org/pub/archive/fedora/linux/releases/20/Everything/x86_64/os/Packages/c/",
        "package_name": "curl-7.32.0-3.fc20.x86_64.rpm",
        "product": "curl",
        "version": "7.32.0",
    },
    {
        "url": "http://www.rpmfind.net/linux/fedora/linux/development/rawhide/Everything/aarch64/os/Packages/c/",
        "package_name": "curl-7.74.0-2.fc34.aarch64.rpm",
        "product": "curl",
        "version": "7.74.0",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

# pylint: disable=too-many-public-methods, too-many-arguments, fixme
"""
CVE-bin-tool tests
"""
import contextlib
import importlib
import io
import os
import shutil
import sys
import tarfile
import tempfile
import unittest
import unittest.mock
from test.test_data import __all__ as all_test_name
from test.utils import LONG_TESTS, download_file

import pytest

from cve_bin_tool.cvedb import CVEDB
from cve_bin_tool.version_scanner import VersionScanner

BINARIES_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)), "binaries")
# load test data
test_data = list(
    map(lambda x: importlib.import_module(f"test.test_data.{x}"), all_test_name)
)
mapping_test_data = map(lambda x: x.mapping_test_data, test_data)
package_test_data = map(lambda x: x.package_test_data, test_data)
DISABLED_TESTS_ACTIONS = ["gimp"]
DISABLED_TESTS_LOCAL = []


class TestScanner:
    """Runs a series of tests against our "faked" binaries.

    The faked binaries are very small c files containing the same string signatures we use
    in the cve-bin-tool.  They should trigger results as if they contained the library and
    version specified in the file name.

    At this time, the tests work only in python3.
    """

    @classmethod
    def setup_class(cls):
        cls.cvedb = CVEDB()
        if os.getenv("UPDATE_DB") == "1":
            cls.cvedb.get_cvelist_if_stale()
        else:
            print("Skip NVD database updates.")
        # Instantiate a scanner
        cls.scanner = VersionScanner(should_extract=True)
        # temp dir for mapping tests
        cls.mapping_test_dir = tempfile.mkdtemp(prefix="mapping-test-")
        # temp dir for tests that require downloads
        cls.package_test_dir = tempfile.mkdtemp(prefix="package_test-")

    @classmethod
    def teardown_class(cls):
        shutil.rmtree(cls.package_test_dir)
        shutil.rmtree(cls.mapping_test_dir)

    def test_false_positive(self):
        self.scanner.all_cves = []
        with tempfile.NamedTemporaryFile(
            "w+b",
            suffix="-test-false-positive.out",
            dir=self.mapping_test_dir,
            delete=False,
        ) as f:
            common_signatures = [
                # common strings generated by a compiler
                b"\x7f\x45\x4c\x46\x02\x01\x01\x03\n",
                b"GCC: (x86_64-posix-seh-rev0, Built by MinGW-W64 project) 8.1.0\n",
                b"GNU C17 8.1.0 -mtune=core2 -march=nocona -g -g -g -O2 -O2 -O2 -fno-ident -fbuilding-libgcc -fno-stack-protector\n",
                b"../../../../../src/gcca-8.1.0/libgcc/libgcc2.c\n",
                rb"C:\mingw810\x86_64-810-posix-seh-rt_v6-rev0\build\gcca-8.1.0\x86_64-w64-mingw32\libgcc\n",
                b"GCC: (Ubuntu 7.5.0-3ubuntu1~18.04) 7.5.0\n",
                # bare version strings.
                b"1_0",
                b"1_2_3",
                b"1.4",
                b"1.2.3",
                b"6.7a",
                b"8.9.10-11",
                b"1-2",
                b"1-2-4",
                b"1.2.3-rc.1",
            ]
            f.writelines(common_signatures)
            filename = f.name
        for params in self.scanner.scan_file(
            os.path.join(self.mapping_test_dir, filename)
        ):
            if params:
                pytest.fail(msg=f"Checker has detected false positive: {params}")

    @pytest.mark.parametrize(
        "product, version, version_strings",
        (
            (d["product"], d["version"], d["version_strings"])
            for list_data in mapping_test_data
            for d in list_data
        ),
    )
    def test_version_mapping(self, product, version, version_strings):
        """Helper function to scan a binary and check that it contains
        certain cves for a version and doesn't contain others."""

        # Run the scan
        version_strings = list(map(lambda s: f"{s}\n".encode("ascii"), version_strings))
        # first filename will test "is" and second will test "contains"
        filenames = [
            f"-{product}-{version}.out",
            f"{'.'.join(list(product))}-{version}.out",
        ]
        for filename in filenames:
            with tempfile.NamedTemporaryFile(
                "w+b", suffix=filename, dir=self.mapping_test_dir, delete=False
            ) as f:
                f.write(b"\x7f\x45\x4c\x46\x02\x01\x01\x03\n")
                f.writelines(version_strings)
                filename = f.name

            list_products = set()
            list_versions = set()
            expected_path = os.path.join(self.mapping_test_dir, filename)
            for scan_info in self.scanner.recursive_scan(expected_path):
                if scan_info:
                    product_info, file_path = scan_info
                    list_products.add(product_info.product)
                    list_versions.add(product_info.version)
                    assert file_path == expected_path
            assert product in list_products
            assert version in list_versions

    def make_condensed_from_download(self, download_path, condensed_path):
        """Extract a downloaded file recursivley, find all the strings for each
        file. Create a new archive with only the strings that were found."""
        dot_extracted = ".extracted"
        file_lines_pairs = []

        class MakeCondensedVersionScanner(VersionScanner):
            def run_checkers(self, filename, lines):
                # Remove until .extracted plus 1 for the '/' after it. This will
                # give us the path to the file relative to the root of the
                # archive it came from
                file_lines_pairs.append(
                    (
                        filename[
                            filename.index(dot_extracted) + len(dot_extracted) + 1 :
                        ],
                        lines,
                    )
                )
                yield None

        # Run the recursive extraction
        for _ in MakeCondensedVersionScanner(should_extract=True).recursive_scan(
            download_path
        ):
            pass

        with contextlib.ExitStack() as stack:
            # gzip will add a last modified time by calling time.time, to ensure
            # that the hash is always the same, we set the time to 0
            stack.enter_context(unittest.mock.patch("time.time", return_value=1))
            # Create the archive
            with tarfile.open(condensed_path, mode="w|gz") as archive:
                for filepath, lines in file_lines_pairs:
                    # Create a bytes object for each file
                    fileobj = stack.enter_context(io.BytesIO())
                    fileobj.write(b"\x7f\x45\x4c\x46\x02\x01\x01\x03\n")
                    fileobj.writelines(
                        list(map(lambda s: f"{s}\n".encode("ascii"), lines))
                    )
                    # If we call write at all we need to seek back to the beginning
                    fileobj.seek(0)
                    # Create the TarInfo objects
                    tarinfo = tarfile.TarInfo(name=filepath)
                    tarinfo.size = len(fileobj.getvalue())
                    # Add the file to the archive
                    archive.addfile(tarinfo, fileobj=fileobj)

    def condensed_filepath(self, url, package_name):
        # Make directories for downloads and to store condensed versions
        downloads_dir = os.path.join(os.path.dirname(__file__), "downloads")
        condensed_dir = os.path.join(os.path.dirname(__file__), "condensed-downloads")
        for dirpath in [downloads_dir, condensed_dir]:
            if not os.path.isdir(dirpath):
                os.mkdir(dirpath)
        # Check if we've already made a condensed version of the file, if we
        # have, we're done.
        condensed_path = os.path.join(condensed_dir, package_name + ".tar.gz")
        if os.path.isfile(condensed_path):
            return condensed_path
        # Download the file if we don't have a condensed version of it and we
        # don't have it downloaded already
        download_path = os.path.join(downloads_dir, package_name)
        if not os.path.isfile(download_path):
            download_file(url + package_name, download_path)
        # Make the condensed version of the file
        self.make_condensed_from_download(download_path, condensed_path)
        # Return the condensed version
        return condensed_path

    @pytest.mark.parametrize(
        "url, package_name, product, version",
        (
            pytest.param(
                d["url"],
                d["package_name"],
                d["product"],
                d["version"],
                marks=[
                    pytest.mark.skipif(
                        "ACTIONS" in os.environ
                        and os.environ["ACTIONS"] == "1"
                        and d["product"] in DISABLED_TESTS_ACTIONS,
                        reason=f"{d['product']} Long tests disabled due to issues working in github actions",
                    ),
                    pytest.mark.skipif(
                        "ACTIONS" not in os.environ
                        and d["product"] in DISABLED_TESTS_LOCAL,
                        reason=f"{d['product']} Long tests disabled locally",
                    ),
                ],
            )
            for list_data in package_test_data
            for d in list_data
        ),
    )
    @unittest.skipUnless(LONG_TESTS() > 0, "Skipping long tests")
    def test_version_in_package(self, url, package_name, product, version):
        """Helper function to get a file (presumed to be a real copy
        of a library, probably from a Linux distribution) and run a
        scan on it.  Any test using this should likely be listed as a
        long test."""
        # get file
        tempfile = self.condensed_filepath(url, package_name)
        # new scanner for the new test.
        # self.scanner = VersionScanner(self.cve_scanner, should_extract=True)
        # run the tests
        list_products = set()
        list_versions = set()
        for scan_info in self.scanner.recursive_scan(tempfile):
            if scan_info:
                product_info, file_path = scan_info
                list_products.add(product_info.product)
                list_versions.add(product_info.version)

        # Make sure the product and version are in the results
        assert (
            product in list_products
        ), f"""{product} not found in {package_name}.
        The checker signature or url may be incorrect."""
        assert version in list_versions

    def test_does_not_scan_symlinks(self):
        """Test that the scanner doesn't scan symlinks"""
        if sys.platform.startswith("linux"):
            # we can only do this in linux since symlink is privilege operation in windows
            os.symlink("non-existant-file", "non-existant-link")
            try:
                with pytest.raises(StopIteration):
                    next(
                        self.scanner.scan_file(
                            os.path.join(self.mapping_test_dir, "non-existant-link")
                        )
                    )
            finally:
                os.unlink("non-existant-link")

    def test_cannot_open_file(self, caplog):
        """Test behaviour when file cannot be opened"""
        with pytest.raises(StopIteration):
            next(
                self.scanner.scan_file(
                    os.path.join(self.mapping_test_dir, "non-existant-file")
                )
            )
        assert str.find("Invalid file", caplog.text)

    def test_clean_file_path(self):
        filepath = (
            "/tmp/cve-bin-tool/dhtei34fd/file_name.extracted/usr/bin/vulnerable_file"
        )
        expected_path = "/usr/bin/vulnerable_file"

        cleaned_path = self.scanner.clean_file_path(filepath)
        assert expected_path == cleaned_path



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

#!python
import re

import pkg_resources
import pytest

from cve_bin_tool.checkers import Checker, VendorProductPair

Pattern = type(re.compile("", 0))


class TestCheckerClass:
    def test_valid_checker(self):
        class MyChecker(Checker):
            CONTAINS_PATTERNS = [r"look"]
            VERSION_PATTERNS = [r"for"]
            FILENAME_PATTERNS = [r"myproduct"]
            VENDOR_PRODUCT = [("myvendor", "myproduct")]

        assert type(MyChecker.CONTAINS_PATTERNS[0]) == Pattern
        assert type(MyChecker.VERSION_PATTERNS[0]) == Pattern
        assert type(MyChecker.FILENAME_PATTERNS[0]) == Pattern
        assert type(MyChecker.VENDOR_PRODUCT[0]) == VendorProductPair

    def test_no_vpkg(self):
        with pytest.raises(AssertionError) as e:

            class MyChecker(Checker):
                CONTAINS_PATTERNS = [r"look"]
                VERSION_PATTERNS = [r"for"]
                FILENAME_PATTERNS = [r"myproduct"]
                PRODUCT_NAME = "mychecker"

        assert e.value.args[0] == "MyChecker needs at least one vendor product pair"


class TestCheckerVersionParser:
    """Run a series of tests directly against individual checkers.
    This is a companion to the tests in TestScanner."""

    @pytest.mark.parametrize(
        "checker_name, file_name, expected_results",
        [
            ("binutils", "dlltool", ["binutils"]),
            ("bzip2", "bzip2.so", ["bzip2"]),
            ("cups", "cupsd.so", ["cups"]),
            ("curl", "libcurl.so.4", ["curl"]),
            ("expat", "libexpat.so", ["expat"]),
            ("ffmpeg", "libffmpeg.so", ["ffmpeg"]),
            ("gnutls_cli", "libgnutls.so", ["gnutls-cli"]),
            ("gnutls_serv", "gnutls-serv", ["gnutls-serv"]),
            ("gstreamer", "libgstreamer.so", ["gstreamer"]),
            ("hostapd", "hostapd.so", ["hostapd"]),
            (
                "icu",
                "international_components_for_unicode.o",
                ["international_components_for_unicode"],
            ),
            ("kerberos", "kerberos", ["kerberos", "kerberos_5"]),
            ("libcurl", "libcurl.so.2.0", ["libcurl"]),
            ("libdb", "libdb-2.0.so", ["libdb"]),
            ("libgcrypt", "libgcrypt.so.1.0", ["libgcrypt"]),
            ("libjpeg", "libjpg.so.2.0", ["libjpeg-turbo"]),
            ("libnss", "libnss.so.1.0", ["nss"]),
            ("libtiff", "libtiff.so.1.0", ["tiff"]),
            ("lighttpd", "lighttpd", ["lighttpd"]),
            ("ncurses", "libform", ["ncurses"]),
            ("nessus", "libnessus", ["nessus"]),
            ("nginx", "nginx", ["nginx"]),
            ("node", "bin/node", ["node"]),
            ("openssh_client", "scp", ["openssh-client"]),
            ("openssh_client", "sftp", ["openssh-client"]),
            ("openssh_client", "ssh", ["openssh-client"]),
            ("openssh_client", "ssh-add", ["openssh-client"]),
            ("openssh_client", "ssh-agent", ["openssh-client"]),
            ("openssh_client", "ssh-argv0", ["openssh-client"]),
            ("openssh_client", "ssh-copy-id", ["openssh-client"]),
            ("openssh_client", "ssh-keygen", ["openssh-client"]),
            ("openssh_client", "ssh-keyscan", ["openssh-client"]),
            ("openssh_client", "slogin", ["openssh-client"]),
            ("openssh_server", "sshd", ["openssh-server"]),
            ("openssl", "libcrypto.so", ["openssl"]),
            ("openswan", "ranbits.so", ["openswan"]),
            ("png", "libpng.so.1.0", ["png"]),
            ("postgresql", "psql", ["postgresql"]),
            ("python", "python", ["python"]),
            ("python", "python2.7", ["python"]),
            ("python", "python3.8", ["python"]),
            ("python", "python3.9", ["python"]),
            ("rsyslog", "rsyslogd.so", ["rsyslog"]),
            ("sqlite", "sqlite3", ["sqlite"]),
            ("strongswan", "libcharon.so", ["strongswan"]),
            ("syslogng", "syslog-ng.so", ["syslog-ng"]),
            ("systemd", "libsystemd.so.0", ["systemd"]),
            ("varnish", "varnish", ["varnish"]),
            ("wireshark", "rawshark", ["wireshark"]),
            ("xerces", "libxerces-c.so", ["xerces"]),
            ("xml2", "libxml2.so.0", ["xml2"]),
            ("zlib", "libz.so.0", ["zlib"]),
        ],
    )
    def test_filename_is(self, checker_name, file_name, expected_results):
        """Test a checker's filename detection"""
        checkers = pkg_resources.iter_entry_points("cve_bin_tool.checker")
        for checker in checkers:
            if checker.name == checker_name:
                Checker = checker.load()
                checker = Checker()

                result = checker.get_version([""], file_name)

                if "is_or_contains" in result:
                    results = [dict()]
                    results[0] = result
                else:
                    results = result

                for result, expected_result in zip(results, expected_results):
                    assert result["is_or_contains"] == "is"

    def test_glibc_latest_version(self):
        """Checker didn't detect latest version of glibc"""

        common_versions = [
            # common strings generated by a compiler
            r"$Pj?PW\n"
            r"D$ P\n"
            r"D$0PW\n"
            r"GNU C Library (GNU libc) release release version\n",
            r"Mandatory or optional arguments to long options are also mandatory or optional for any corresponding short options.\n",
            r"Compiled by GNU CC version 10.2.1 20200804 (Red Hat 10.2.1-2).\n",
            # bare version strings.
            r"GLIBC_2.0\n",
            r"GLIBC_2.1\n",
            r"GLIBC_2.1.1\n",
            r"GLIBC_2.3\n",
            r"GLIBC_2.3.2\n",
            r"GLIBC_2.12\n",
            r"GLIBC_2.23\n",
            r"GLIBC_2.30\n",
            r"GLIBC_2.32\n",
        ]
        versions_not_considered = [
            "2.0",
            "2.1",
            "2.1.1",
            "2.3.2",
            "2.12",
            "2.23",
            "2.30",
        ]
        file_name = "libc.so.6"
        checkers = pkg_resources.iter_entry_points("cve_bin_tool.checker")
        result = None
        for checker in checkers:
            if checker.name == "glibc":
                Checker = checker.load()
                checker = Checker()

                result = checker.get_version(common_versions, file_name)

        if "is_or_contains" in result:
            assert result["version"] not in versions_not_considered



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "libtiff",
        "version": "4.0.2",
        "version_strings": [
            'TIFF directory is missing required \\"StripByteCounts\\" field, calculating from imagelength',
            "LIBTIFF, Version 4.0.2",
        ],
    }
]
package_test_data = [
    {
        "url": "https://archives.fedoraproject.org/pub/archive/fedora/linux/releases/30/Everything/x86_64/os/Packages/l/",
        "package_name": "libtiff-4.0.10-4.fc30.i686.rpm",
        "product": "libtiff",
        "version": "4.0.10",
    },
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "libtiff-4.0.3-35.el7.x86_64.rpm",
        "product": "libtiff",
        "version": "4.0.3",
    },
]
